@@ -12,7 +12,7 @@ distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
-]]--
+]] --
 
 --[[[
 -- @module lua_util
@@ -32,6 +32,37 @@ local nospace = 1 - space
 local ptrim = space ^ 0 * lpeg.C((space ^ 0 * nospace ^ 1) ^ 0)
 local match = lpeg.match
 
+local function shallowcopy(orig)
+  local orig_type = type(orig)
+  local copy
+  if orig_type == 'table' then
+    copy = {}
+    for orig_key, orig_value in pairs(orig) do
+      copy[orig_key] = orig_value
+    end
+  else
+    copy = orig
+  end
+  return copy
+end
+local function deepcopy(orig)
+  local orig_type = type(orig)
+  local copy
+  if orig_type == 'table' then
+    copy = {}
+    for orig_key, orig_value in next, orig, nil do
+      copy[deepcopy(orig_key)] = deepcopy(orig_value)
+    end
+    if getmetatable(orig) then
+      setmetatable(copy, deepcopy(getmetatable(orig)))
+    end
+  else
+    -- number, string, boolean, etc
+    copy = orig
+  end
+  return copy
+end
+
 lupa.configure('{%', '%}', '{=', '=}', '{#', '#}', {
   keep_trailing_newline = true,
   autoescape = false,
@@ -42,6 +73,10 @@ lupa.filters.pbkdf = function(s)
   return cr.pbkdf(s)
 end
 
+-- Dirty hacks to avoid shared state
+package.loaded['lupa'] = nil
+local lupa_orig = require "lupa"
+
 local function rspamd_str_split(s, sep)
   local gr
   if not sep then
@@ -61,7 +96,7 @@ local function rspamd_str_split(s, sep)
       if type(sep) == 'string' then
         _sep = lpeg.S(sep) -- Assume set
       else
-        _sep = sep -- Assume lpeg object
+        _sep = sep         -- Assume lpeg object
       end
       local elem = lpeg.C((1 - _sep) ^ 0)
       local p = lpeg.Ct(elem * (_sep * elem) ^ 0)
@@ -157,48 +192,88 @@ exports.template = function(tmpl, keys)
 end
 
 local function enrich_template_with_globals(env)
-  local newenv = exports.shallowcopy(env)
+  local newenv = shallowcopy(env)
   newenv.paths = rspamd_paths
   newenv.env = rspamd_env
 
   return newenv
 end
 --[[[
--- @function lua_util.jinja_template(text, env[, skip_global_env])
+-- @function lua_util.jinja_template(text, env[, skip_global_env][, is_orig][, custom_filters])
 -- Replaces values in a text template according to jinja2 syntax
 -- @param {string} text text containing variables
 -- @param {table} replacements key/value pairs for replacements
 -- @param {boolean} skip_global_env don't export Rspamd superglobals
+-- @param {boolean} is_orig use the original lupa configuration with {% raw %}`{{`{% endraw %} for variables
+-- @param {table} custom_filters custom filters to use (or nil if not needed)
 -- @return {string} string containing replaced values
 -- @example
--- lua_util.jinja_template("HELLO {{FOO}} {{BAR}}!", {['FOO'] = 'LUA', ['BAR'] = 'WORLD'})
+-- lua_util.jinja_template("HELLO {=FOO=} {=BAR=}!", {['FOO'] = 'LUA', ['BAR'] = 'WORLD'})
 -- "HELLO LUA WORLD!"
 --]]
-exports.jinja_template = function(text, env, skip_global_env)
+exports.jinja_template = function(text, env, skip_global_env, is_orig, custom_filters)
+  local lupa_to_use = is_orig and lupa_orig or lupa
   if not skip_global_env then
     env = enrich_template_with_globals(env)
   end
 
-  return lupa.expand(text, env)
+  local orig_filters = {}
+  if type(custom_filters) == 'table' then
+    for k, v in pairs(custom_filters) do
+      orig_filters[k] = lupa_to_use.filters[k]
+      lupa_to_use.filters[k] = v
+    end
+  end
+
+  local result = lupa_to_use.expand(text, env)
+
+  -- Restore custom filters
+  if type(custom_filters) == 'table' then
+    for k, _ in pairs(custom_filters) do
+      lupa_to_use.filters[k] = orig_filters[k]
+    end
+  end
+
+  return result
 end
 
 --[[[
--- @function lua_util.jinja_file(filename, env[, skip_global_env])
+-- @function lua_util.jinja_file(filename, env[, skip_global_env][, is_orig][, custom_filters])
 -- Replaces values in a text template according to jinja2 syntax
 -- @param {string} filename name of file to expand
 -- @param {table} replacements key/value pairs for replacements
 -- @param {boolean} skip_global_env don't export Rspamd superglobals
+-- @param {boolean} is_orig use the original lupa configuration with {% raw %}`{{`{% endraw %} for variables
+-- @param {table} custom_filters custom filters to use (or nil if not needed)
 -- @return {string} string containing replaced values
 -- @example
--- lua_util.jinja_template("HELLO {{FOO}} {{BAR}}!", {['FOO'] = 'LUA', ['BAR'] = 'WORLD'})
+-- lua_util.jinja_template("HELLO {=FOO=} {=BAR=}!", {['FOO'] = 'LUA', ['BAR'] = 'WORLD'})
 -- "HELLO LUA WORLD!"
 --]]
-exports.jinja_template_file = function(filename, env, skip_global_env)
+exports.jinja_template_file = function(filename, env, skip_global_env, is_orig, custom_filters)
+  local lupa_to_use = is_orig and lupa_orig or lupa
   if not skip_global_env then
     env = enrich_template_with_globals(env)
   end
 
-  return lupa.expand_file(filename, env)
+  local orig_filters = {}
+  if type(custom_filters) == 'table' then
+    for k, v in pairs(custom_filters) do
+      orig_filters[k] = lupa_to_use.filters[k]
+      lupa_to_use.filters[k] = v
+    end
+  end
+
+  local result = lupa_to_use.expand_file(filename, env)
+
+  -- Restore custom filters
+  if type(custom_filters) == 'table' then
+    for k, _ in pairs(custom_filters) do
+      lupa_to_use.filters[k] = orig_filters[k]
+    end
+  end
+
+  return result
 end
 
 exports.remove_email_aliases = function(email_addr)
@@ -314,8 +389,8 @@ end
 --]]
 
 local unpack_function = table.unpack or unpack
-exports.unpack = function(t)
-  return unpack_function(t)
+exports.unpack = function(...)
+  return unpack_function(...)
 end
 
 --[[[
@@ -492,14 +567,14 @@ local function parse_time_interval(str)
   parser.fractional = (lpeg.P(".")) *
       (digit ^ 1)
   parser.number = (parser.integer *
-      (parser.fractional ^ -1)) +
+        (parser.fractional ^ -1)) +
       (lpeg.S("+-") * parser.fractional)
   parser.time = lpeg.Cf(lpeg.Cc(1) *
-      (parser.number / tonumber) *
-      ((lpeg.S("smhdwy") / parse_time_suffix) ^ -1),
-      function(acc, val)
-        return acc * val
-      end)
+    (parser.number / tonumber) *
+    ((lpeg.S("smhdwy") / parse_time_suffix) ^ -1),
+    function(acc, val)
+      return acc * val
+    end)
 
   local t = lpeg.match(parser.time, str)
 
@@ -540,14 +615,14 @@ local function dehumanize_number(str)
   parser.fractional = (lpeg.P(".")) *
       (digit ^ 1)
   parser.number = (parser.integer *
-      (parser.fractional ^ -1)) +
+        (parser.fractional ^ -1)) +
       (lpeg.S("+-") * parser.fractional)
   parser.humanized_number = lpeg.Cf(lpeg.Cc(1) *
-      (parser.number / tonumber) *
-      (((lpeg.S("kmg") * (lpeg.P("b") ^ -1)) / parse_suffix) ^ -1),
-      function(acc, val)
-        return acc * val
-      end)
+    (parser.number / tonumber) *
+    (((lpeg.S("kmg") * (lpeg.P("b") ^ -1)) / parse_suffix) ^ -1),
+    function(acc, val)
+      return acc * val
+    end)
 
   local t = lpeg.match(parser.humanized_number, str)
 
@@ -653,7 +728,6 @@ exports.table_merge = table_merge
 -- Performs header folding
 --]]
 exports.fold_header = function(task, name, value, stop_chars)
-
   local how
 
   if task:has_flag("milter") then
@@ -665,6 +739,41 @@ exports.fold_header = function(task, name, value, stop_chars)
   return rspamd_util.fold_header(name, value, how, stop_chars)
 end
 
+--[[[
+-- @function lua_util.fold_header_with_encoding(task, name, value[, opts])
+-- Folds header value using name and optionally encodes the result.
+-- Encoding policy defaults to 'auto':
+-- - If MIME UTF-8 is enabled, encode only when the folded value is not valid UTF-8
+-- - Otherwise, always encode
+-- `opts` table fields:
+--   * stop_chars: optional string with extra fold-on characters
+--   * encode: true|false|'auto' (default: 'auto')
+--   * structured: boolean, pass as `is_structured` to mime_header_encode (default: false)
+-- @return {string} folded (and possibly encoded) header value
+--]]
+exports.fold_header_with_encoding = function(task, name, value, opts)
+  opts = opts or {}
+  local stop_chars = opts.stop_chars
+  local encode = opts.encode
+  local structured = opts.structured or false
+
+  local folded = exports.fold_header(task, name, value, stop_chars)
+
+  if encode == nil or encode == 'auto' then
+    if rspamd_config:is_mime_utf8() then
+      if not rspamd_util.is_valid_utf8(folded) then
+        folded = rspamd_util.mime_header_encode(folded, structured)
+      end
+    else
+      folded = rspamd_util.mime_header_encode(folded, structured)
+    end
+  elseif encode == true then
+    folded = rspamd_util.mime_header_encode(folded, structured)
+  end
+
+  return folded
+end
+
 --[[[
 -- @function lua_util.override_defaults(defaults, override)
 -- Overrides values from defaults with override
@@ -680,9 +789,29 @@ local function override_defaults(def, override)
 
   local res = {}
 
-  for k, v in pairs(override) do
-    if type(v) == 'table' then
-      if def[k] and type(def[k]) == 'table' then
+  -- Allow transparent ucl
+  local pairs_func, def_pairs_func = pairs, pairs
+  local type_func, def_type_func = type, type
+  if type(override[0]) == 'userdata' then
+    pairs_func = function(t)
+      return t:pairs()
+    end
+    type_func = function(t)
+      return t:type()
+    end
+  end
+  if type(def[0]) == 'userdata' then
+    def_pairs_func = function(t)
+      return t:pairs()
+    end
+    def_type_func = function(t)
+      return t:type()
+    end
+  end
+
+  for k, v in pairs_func(override) do
+    if type_func(v) == 'table' then
+      if def[k] and def_type_func(def[k]) == 'table' then
         -- Recursively override elements
         res[k] = override_defaults(def[k], v)
       else
@@ -693,7 +822,7 @@ local function override_defaults(def, override)
     end
   end
 
-  for k, v in pairs(def) do
+  for k, v in def_pairs_func(def) do
     if type(res[k]) == 'nil' then
       res[k] = v
     end
@@ -727,9 +856,9 @@ exports.filter_specific_urls = function(urls, params)
       cache_key = params.prefix
     else
       cache_key = string.format('sp_urls_%d%s%s%s', params.limit,
-          tostring(params.need_emails or false),
-          tostring(params.need_images or false),
-          tostring(params.need_content or false))
+        tostring(params.need_emails or false),
+        tostring(params.need_images or false),
+        tostring(params.need_content or false))
     end
     local cached = params.task:cache_get(cache_key)
 
@@ -874,7 +1003,6 @@ exports.filter_specific_urls = function(urls, params)
         -- Prefer less urls to more urls per esld
         return #e1 < #e2
       end
-
     end)
 
     return tbl
@@ -896,7 +1024,6 @@ exports.filter_specific_urls = function(urls, params)
           item_found = true
         end
       end
-
     until limit <= 0 or not item_found
 
     res = exports.values(res)
@@ -985,7 +1112,7 @@ exports.extract_specific_urls = function(params_or_task, lim, need_emails, filte
     emails = params.need_emails,
     images = params.need_images,
     content = params.need_content,
-    flags = params.flags, -- maybe nil
+    flags = params.flags,           -- maybe nil
     flags_mode = params.flags_mode, -- maybe nil
   }
 
@@ -1000,9 +1127,9 @@ exports.extract_specific_urls = function(params_or_task, lim, need_emails, filte
         cache_key_suffix = table.concat(params.flags) .. (params.flags_mode or '')
       else
         cache_key_suffix = string.format('%s%s%s',
-            tostring(params.need_emails or false),
-            tostring(params.need_images or false),
-            tostring(params.need_content or false))
+          tostring(params.need_emails or false),
+          tostring(params.need_images or false),
+          tostring(params.need_content or false))
       end
       cache_key = string.format('sp_urls_%d%s', params.limit, cache_key_suffix)
     end
@@ -1019,6 +1146,7 @@ exports.extract_specific_urls = function(params_or_task, lim, need_emails, filte
   return exports.filter_specific_urls(urls, params)
 end
 
+
 --[[[
 -- @function lua_util.deepcopy(table)
 -- params: {
@@ -1026,24 +1154,6 @@ end
 -- }
 -- Performs deep copy of the table. Including metatables
 --]]
-local function deepcopy(orig)
-  local orig_type = type(orig)
-  local copy
-  if orig_type == 'table' then
-    copy = {}
-    for orig_key, orig_value in next, orig, nil do
-      copy[deepcopy(orig_key)] = deepcopy(orig_value)
-    end
-    if getmetatable(orig) then
-      setmetatable(copy, deepcopy(getmetatable(orig)))
-    end
-  else
-    -- number, string, boolean, etc
-    copy = orig
-  end
-  return copy
-end
-
 exports.deepcopy = deepcopy
 
 --[[[
@@ -1077,19 +1187,7 @@ exports.deepsort = deepsort
 -- @function lua_util.shallowcopy(tbl)
 -- Performs shallow (and fast) copy of a table or another Lua type
 --]]
-exports.shallowcopy = function(orig)
-  local orig_type = type(orig)
-  local copy
-  if orig_type == 'table' then
-    copy = {}
-    for orig_key, orig_value in pairs(orig) do
-      copy[orig_key] = orig_value
-    end
-  else
-    copy = orig
-  end
-  return copy
-end
+exports.shallowcopy = shallowcopy
 
 -- Debugging support
 local logger = require "rspamd_logger"
@@ -1122,7 +1220,7 @@ exports.init_debug_logging = function(config)
           if debug_modules[mod] then
             debug_modules[alias] = true
             logger.infox(config, 'enable debug for Lua module %s (%s aliased)',
-                alias, mod)
+              alias, mod)
           end
         end
       end
@@ -1168,7 +1266,7 @@ exports.add_debug_alias = function(mod, alias)
   if debug_modules[mod] then
     debug_modules[alias] = true
     logger.infox(rspamd_config, 'enable debug for Lua module %s (%s aliased)',
-        alias, mod)
+      alias, mod)
   end
 end
 ---[[[
@@ -1225,6 +1323,85 @@ exports.maybe_obfuscate_string = function(subject, settings, prefix)
   return subject
 end
 
+---[[[
+-- @function lua_util.maybe_encrypt_header(header, settings, prefix)
+-- Encode header with configured public key if enabled in settings.
+-- If header is not set then nil is returned. If pub_key is empty then header is returned.
+-- Supported settings:
+-- * <prefix>_encrypt = false - no need for encryption of a header
+-- * <prefix>_key = 'key' - key that is used encrypt header
+-- * <prefix>_nonce = 'nonce' - nonce to encrypt header(optional)
+-- @return encrypted header
+---]]]
+exports.maybe_encrypt_header = function(header, settings, prefix)
+  local rspamd_secretbox = require "rspamd_cryptobox_secretbox"
+
+  if not header or header == '' then
+    logger.errx(rspamd_config, "Header is empty or nil. Header: %s", header)
+    return nil
+  elseif settings[prefix .. '_encrypt'] then
+    local key = settings[prefix .. '_key']
+    if not key or key == '' then
+      logger.errx(rspamd_config, "Key is empty or nil. Key: %s", key)
+      return header
+    end
+    local cryptobox = rspamd_secretbox.create(key)
+
+    local nonce = settings[prefix .. '_nonce']
+    local encrypted_header = ''
+    if not nonce or nonce == '' then
+      encrypted_header, nonce = cryptobox:encrypt(header)
+    else
+      encrypted_header = cryptobox:encrypt(header, nonce)
+    end
+    encrypted_header = nonce .. encrypted_header
+    return encrypted_header
+  end
+end
+
+---[[[
+-- @function lua_util.maybe_decrypt_header(header, settings, prefix, nonce)
+-- Decode enoced with configured public_key header if enabled in settings.
+-- If encoded header is not set then nil is returned. If pub_key is empty then encoded header is returned.
+-- Supported settings:
+-- * <prefix>_encrypt = false - no need for decryption of a header
+-- * <prefix>_key = 'key' - key that is used decrypt header
+-- * <prefix>_nonce = 'nonce' - nonce used to encrypt header(optional)
+-- Nonce is an optional argument if <prefix>_nonce is provided, otherwise it is an required argument
+-- and <prefix>_nonce is an optional
+-- @return decrypted header
+---]]]
+exports.maybe_decrypt_header = function(encrypted_header, settings, prefix, nonce)
+  local rspamd_secretbox = require "rspamd_cryptobox_secretbox"
+
+  if not encrypted_header or encrypted_header == '' then
+    logger.errx(rspamd_config, "Encrypted header is empty or nil. Encrypted header: %s", encrypted_header)
+    return nil
+  elseif settings[prefix .. '_encrypt'] then
+    local key = settings[prefix .. '_key']
+    if not key or key == '' then
+      logger.errx(rspamd_config, "Key is empty or nil. Key: %s", key)
+      return encrypted_header
+    end
+    local cryptobox = rspamd_secretbox.create(key)
+
+    local result = false
+    local header = ''
+    if not nonce then
+      result, header = cryptobox:decrypt(encrypted_header, settings[prefix .. '_nonce'])
+    else
+      result, header = cryptobox:decrypt(encrypted_header, nonce)
+    end
+
+    if not result then
+      logger.infox(rspamd_config, "Decryption is failed with result: %s and decrypted header: %s", result, header)
+      return nil
+    end
+
+    return header
+  end
+end
+
 ---[[[
 -- @function lua_util.callback_from_string(str)
 -- Converts a string like `return function(...) end` to lua function and return true and this function
@@ -1476,7 +1653,7 @@ exports.is_skip_local_or_authed = function(task, conf, ip)
     conf = { false, false }
   end
   if ((not conf[2] and task:get_user()) or
-      (not conf[1] and type(ip) == 'userdata' and ip:is_local())) then
+        (not conf[1] and type(ip) == 'userdata' and ip:is_local())) then
     return true
   end
 
@@ -1621,6 +1798,30 @@ local function join_path(...)
 end
 exports.join_path = join_path
 
+---[[[
+-- @function lua_util.url_encode_string(str)
+-- URL encodes a string
+--
+-- @param {string} str string to encode
+-- @return {string} URL encoded string
+--
+---]]]
+local function url_encode_string(str)
+  if str == nil then
+    return ''
+  end
+  str = string.gsub(str, "([^%w _%%%-%.~])",
+    function(c)
+      return string.format("%%%02X", string.byte(c))
+    end)
+  str = string.gsub(str, " ", "+")
+  return str
+end
+exports.url_encode_string = url_encode_string
+
+assert(url_encode_string('上海+中國') == '%E4%B8%8A%E6%B5%B7%2B%E4%B8%AD%E5%9C%8B')
+assert(url_encode_string('? and the Mysterians') == '%3F+and+the+Mysterians')
+
 -- Short unit test for sanity
 if path_sep == '/' then
   assert(join_path('/path', 'to', 'file') == '/path/to/file')
@@ -1630,10 +1831,61 @@ end
 
 -- Defines symbols priorities for common usage in prefilters/postfilters
 exports.symbols_priorities = {
-  top = 10, -- Symbols must be executed first (or last), such as settings
-  high = 9, -- Example: asn
+  top = 10,   -- Symbols must be executed first (or last), such as settings
+  high = 9,   -- Example: asn
   medium = 5, -- Everything should use this as default
   low = 0,
 }
 
+---[[[
+-- @function lua_util.table_to_multipart_body(tbl, boundary)
+-- Converts a key-value map to the table representing multipart body, with the following values:
+-- `data`: data of the part
+-- `filename`: optional filename
+-- `content-type`: content type of the element (optional)
+-- `content-transfer-encoding`: optional CTE header
+local function table_to_multipart_body(tbl, boundary)
+  local seen_data = false
+  local out = {}
+
+  for k, v in pairs(tbl) do
+    if v.data then
+      seen_data = true
+      table.insert(out, string.format('--%s\r\n', boundary))
+      if v.filename then
+        table.insert(out,
+          string.format('Content-Disposition: form-data; name="%s"; filename="%s"\r\n',
+            k, v.filename))
+      else
+        table.insert(out,
+          string.format('Content-Disposition: form-data; name="%s"\r\n', k))
+      end
+      if v['content-type'] then
+        table.insert(out,
+          string.format('Content-Type: %s\r\n', v['content-type']))
+      else
+        table.insert(out, 'Content-Type: text/plain\r\n')
+      end
+      if v['content-transfer-encoding'] then
+        table.insert(out,
+          string.format('Content-Transfer-Encoding: %s\r\n',
+            v['content-transfer-encoding']))
+      else
+        table.insert(out, 'Content-Transfer-Encoding: binary\r\n')
+      end
+      table.insert(out, '\r\n')
+      table.insert(out, v.data)
+      table.insert(out, '\r\n')
+    end
+  end
+
+  if seen_data then
+    table.insert(out, string.format('--%s--\r\n', boundary))
+  end
+
+  return out
+end
+
+exports.table_to_multipart_body = table_to_multipart_body
+
 return exports
