@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,9 +36,11 @@
 #include <unicode/uchar.h>
 #include "sodium.h"
 #include "libserver/cfg_file_private.h"
-#include "lua/lua_common.h"
+#define RSPAMD_TOKENIZER_INTERNAL
 #include "contrib/uthash/utlist.h"
 #include "contrib/t1ha/t1ha.h"
+#include "mime_parser.h"
+#include "libstat/tokenizers/custom_tokenizer.h"
 #include "received.h"
 
 #define GTUBE_SYMBOL "GTUBE"
@@ -71,14 +73,14 @@ rspamd_mime_part_extract_words(struct rspamd_task *task,
 	rspamd_stat_token_t *w;
 	unsigned int i, total_len = 0, short_len = 0;
 
-	if (part->utf_words) {
-		rspamd_stem_words(part->utf_words, task->task_pool, part->language,
+	if (part->utf_words.a) {
+		rspamd_stem_words(&part->utf_words, task->task_pool, part->language,
 						  task->lang_det);
 
-		for (i = 0; i < part->utf_words->len; i++) {
+		for (i = 0; i < kv_size(part->utf_words); i++) {
 			uint64_t h;
 
-			w = &g_array_index(part->utf_words, rspamd_stat_token_t, i);
+			w = &kv_A(part->utf_words, i);
 
 			if (w->stemmed.len > 0) {
 				/*
@@ -108,7 +110,7 @@ rspamd_mime_part_extract_words(struct rspamd_task *task,
 			}
 		}
 
-		if (part->utf_words->len) {
+		if (kv_size(part->utf_words)) {
 			double *avg_len_p, *short_len_p;
 
 			avg_len_p = rspamd_mempool_get_variable(task->task_pool,
@@ -185,21 +187,24 @@ rspamd_mime_part_create_words(struct rspamd_task *task,
 		tok_type = RSPAMD_TOKENIZE_RAW;
 	}
 
-	part->utf_words = rspamd_tokenize_text(
+	/* Initialize kvec for words */
+	kv_init(part->utf_words);
+
+	rspamd_tokenize_text(
 		part->utf_stripped_content->data,
 		part->utf_stripped_content->len,
 		&part->utf_stripped_text,
 		tok_type, task->cfg,
 		part->exceptions,
 		NULL,
-		NULL,
+		&part->utf_words,
 		task->task_pool);
 
 
-	if (part->utf_words) {
+	if (part->utf_words.a) {
 		part->normalized_hashes = g_array_sized_new(FALSE, FALSE,
-													sizeof(uint64_t), part->utf_words->len);
-		rspamd_normalize_words(part->utf_words, task->task_pool);
+													sizeof(uint64_t), kv_size(part->utf_words));
+		rspamd_normalize_words(&part->utf_words, task->task_pool);
 	}
 }
 
@@ -209,7 +214,7 @@ rspamd_mime_part_detect_language(struct rspamd_task *task,
 {
 	struct rspamd_lang_detector_res *lang;
 
-	if (!IS_TEXT_PART_EMPTY(part) && part->utf_words && part->utf_words->len > 0 &&
+	if (!IS_TEXT_PART_EMPTY(part) && part->utf_words.a && kv_size(part->utf_words) > 0 &&
 		task->lang_det) {
 		if (rspamd_language_detector_detect(task, task->lang_det, part)) {
 			lang = g_ptr_array_index(part->languages, 0);
@@ -786,6 +791,158 @@ rspamd_message_process_html_text_part(struct rspamd_task *task,
 		text_part->mime_part->urls,
 		task->cfg ? task->cfg->enable_css_parser : true,
 		cur_url_order);
+
+	/* Wire aggregated HTML features */
+	text_part->html_features = (struct rspamd_html_features *) rspamd_html_get_features(text_part->html);
+
+	/* Optionally call CTA/affiliation Lua hook with capped candidates */
+	if (task->cfg && task->cfg->lua_state) {
+		lua_State *L = task->cfg->lua_state;
+		int old_top = lua_gettop(L);
+		if (rspamd_lua_require_function(L, "lua_cta", "process_html_links")) {
+			/* Build ctx table with summary and limited candidates */
+			lua_pushcfunction(L, &rspamd_lua_traceback);
+			int err_idx = lua_gettop(L);
+			lua_pushvalue(L, -2); /* function */
+
+			/* Arg1: task */
+			struct rspamd_task **ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
+			rspamd_lua_setclass(L, rspamd_task_classname, -1);
+			*ptask = task;
+			/* Arg2: text part */
+			struct rspamd_mime_text_part **ptxt = lua_newuserdata(L, sizeof(struct rspamd_mime_text_part *));
+			rspamd_lua_setclass(L, rspamd_textpart_classname, -1);
+			*ptxt = text_part;
+			/* Arg3: ctx table */
+			lua_createtable(L, 0, 4);
+			/* first_party_etld1 if any */
+			if (text_part->html && text_part->html_features) {
+				/* Expose as string if derived */
+				/* Not directly accessible; skip for now, Lua can derive from From: */
+			}
+			/* Summary counters */
+			lua_pushstring(L, "links_total");
+			lua_pushinteger(L, (lua_Integer) text_part->html_features->links.total_links);
+			lua_settable(L, -3);
+			lua_pushstring(L, "domains_total");
+			lua_pushinteger(L, (lua_Integer) text_part->html_features->links.domains_total);
+			lua_settable(L, -3);
+			lua_pushstring(L, "max_links_single_domain");
+			lua_pushinteger(L, (lua_Integer) text_part->html_features->links.max_links_single_domain);
+			lua_settable(L, -3);
+			/* candidates array */
+			lua_pushstring(L, "candidates");
+			int max_candidates = 24; /* TODO: make configurable */
+			lua_createtable(L, max_candidates, 0);
+			int nadded = 0;
+			if (text_part->mime_part && text_part->mime_part->urls && text_part->mime_part->urls->len > 0) {
+				unsigned int i;
+				for (i = 0; i < text_part->mime_part->urls->len && nadded < (unsigned) max_candidates; i++) {
+					struct rspamd_url *u = g_ptr_array_index(text_part->mime_part->urls, i);
+					if (!u) continue;
+					/* filter: only http/https, visible */
+					if (!(u->protocol == PROTOCOL_HTTP || u->protocol == PROTOCOL_HTTPS)) continue;
+					if (u->flags & RSPAMD_URL_FLAG_INVISIBLE) continue;
+					/* Build small table */
+					lua_createtable(L, 0, 8);
+					/* host */
+					lua_pushstring(L, "host");
+					if (u->hostlen > 0) lua_pushlstring(L, rspamd_url_host_unsafe(u), u->hostlen);
+					else
+						lua_pushnil(L);
+					lua_settable(L, -3);
+					/* flags */
+					lua_pushstring(L, "idn");
+					lua_pushboolean(L, !!(u->flags & RSPAMD_URL_FLAG_IDN));
+					lua_settable(L, -3);
+					lua_pushstring(L, "numeric");
+					lua_pushboolean(L, !!(u->flags & RSPAMD_URL_FLAG_NUMERIC));
+					lua_settable(L, -3);
+					lua_pushstring(L, "has_port");
+					lua_pushboolean(L, !!(u->flags & RSPAMD_URL_FLAG_HAS_PORT));
+					lua_settable(L, -3);
+					lua_pushstring(L, "has_query");
+					lua_pushboolean(L, !!(u->flags & RSPAMD_URL_FLAG_QUERY));
+					lua_settable(L, -3);
+					lua_pushstring(L, "display_mismatch");
+					lua_pushboolean(L, (u->ext && u->ext->linked_url && u->ext->linked_url != u));
+					lua_settable(L, -3);
+					/* order */
+					lua_pushstring(L, "order");
+					lua_pushinteger(L, (lua_Integer) u->order);
+					lua_settable(L, -3);
+					lua_pushstring(L, "part_order");
+					lua_pushinteger(L, (lua_Integer) u->part_order);
+					lua_settable(L, -3);
+					/* cta weight from C heuristics */
+					lua_pushstring(L, "weight");
+					lua_pushnumber(L, (lua_Number) rspamd_html_url_button_weight(text_part->html, u));
+					lua_settable(L, -3);
+					/* etld1 computed in Lua if needed */
+					lua_rawseti(L, -2, ++nadded);
+				}
+			}
+			lua_settable(L, -3); /* ctx.candidates = [...] */
+
+			if (lua_pcall(L, 3, 1, err_idx) != 0) {
+				msg_debug_task("lua_cta.process_html_links error: %s", lua_tostring(L, -1));
+			}
+			else if (lua_istable(L, -1)) {
+				/* read result and expose mempool variables */
+				lua_pushstring(L, "cta_affiliated");
+				lua_gettable(L, -2);
+				if (lua_isboolean(L, -1)) {
+					static int val;
+					val = !!lua_toboolean(L, -1);
+					rspamd_mempool_set_variable(task->task_pool, "html_cta_affiliated", &val, NULL);
+				}
+				lua_pop(L, 1);
+				lua_pushstring(L, "cta_weight");
+				lua_gettable(L, -2);
+				if (lua_isnumber(L, -1)) {
+					static double w;
+					w = lua_tonumber(L, -1);
+					rspamd_mempool_set_variable(task->task_pool, "html_cta_weight", &w, NULL);
+				}
+				lua_pop(L, 1);
+				/* If no weight set by Lua, derive from C heuristic */
+				if (!rspamd_mempool_get_variable(task->task_pool, "html_cta_weight") &&
+					text_part->html && text_part->mime_part && text_part->mime_part->urls) {
+					double best_w = 0.0;
+					unsigned int ui;
+					for (ui = 0; ui < text_part->mime_part->urls->len; ui++) {
+						struct rspamd_url *u = g_ptr_array_index(text_part->mime_part->urls, ui);
+						if (!u) continue;
+						if (!(u->protocol == PROTOCOL_HTTP || u->protocol == PROTOCOL_HTTPS)) continue;
+						if (u->flags & RSPAMD_URL_FLAG_INVISIBLE) continue;
+						float cw = rspamd_html_url_button_weight(text_part->html, u);
+						if (cw > best_w) best_w = cw;
+					}
+					if (best_w > 0.0) {
+						rspamd_mempool_set_variable(task->task_pool, "html_cta_weight", &best_w, NULL);
+					}
+				}
+				lua_pushstring(L, "affiliated_ratio");
+				lua_gettable(L, -2);
+				if (lua_isnumber(L, -1)) {
+					static double r;
+					r = lua_tonumber(L, -1);
+					rspamd_mempool_set_variable(task->task_pool, "html_affiliated_links_ratio", &r, NULL);
+				}
+				lua_pop(L, 1);
+				lua_pushstring(L, "trackerish_ratio");
+				lua_gettable(L, -2);
+				if (lua_isnumber(L, -1)) {
+					static double tr;
+					tr = lua_tonumber(L, -1);
+					rspamd_mempool_set_variable(task->task_pool, "html_trackerish_ratio", &tr, NULL);
+				}
+				lua_pop(L, 1);
+			}
+
+			lua_settop(L, old_top);
+		}
+	}
 	rspamd_html_get_parsed_content(text_part->html, &text_part->utf_content);
 
 	if (text_part->utf_content.len == 0) {
@@ -984,8 +1141,38 @@ rspamd_message_from_data(struct rspamd_task *task, const unsigned char *start,
 	else if (task->cfg && task->cfg->libs_ctx) {
 		lua_State *L = task->cfg->lua_state;
 
-		if (rspamd_lua_require_function(L,
-										"lua_magic", "detect_mime_part")) {
+		if (task->cfg->mime_parser_cfg &&
+			rspamd_mime_parser_get_lua_magic_cbref(task->cfg->mime_parser_cfg) != -1) {
+			struct rspamd_mime_part **pmime;
+			struct rspamd_task **ptask;
+
+			lua_rawgeti(L, LUA_REGISTRYINDEX, rspamd_mime_parser_get_lua_magic_cbref(task->cfg->mime_parser_cfg));
+			pmime = lua_newuserdata(L, sizeof(struct rspamd_mime_part *));
+			rspamd_lua_setclass(L, rspamd_mimepart_classname, -1);
+			*pmime = part;
+			ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
+			rspamd_lua_setclass(L, rspamd_task_classname, -1);
+			*ptask = task;
+
+			if (lua_pcall(L, 2, 2, 0) != 0) {
+				msg_err_task("cannot detect type: %s", lua_tostring(L, -1));
+			}
+			else {
+				if (lua_istable(L, -1)) {
+					lua_pushstring(L, "ct");
+					lua_gettable(L, -2);
+
+					if (lua_isstring(L, -1)) {
+						mb = rspamd_mempool_strdup(task->task_pool,
+												   lua_tostring(L, -1));
+					}
+				}
+			}
+
+			lua_settop(L, 0);
+		}
+		else if (rspamd_lua_require_function(L,
+											 "lua_magic", "detect_mime_part")) {
 
 			struct rspamd_mime_part **pmime;
 			struct rspamd_task **ptask;
@@ -1106,8 +1293,8 @@ rspamd_message_dtor(struct rspamd_message *msg)
 
 	PTR_ARRAY_FOREACH(msg->text_parts, i, tp)
 	{
-		if (tp->utf_words) {
-			g_array_free(tp->utf_words, TRUE);
+		if (tp->utf_words.a) {
+			kv_destroy(tp->utf_words);
 		}
 		if (tp->normalized_hashes) {
 			g_array_free(tp->normalized_hashes, TRUE);
@@ -1152,6 +1339,10 @@ rspamd_message_parse(struct rspamd_task *task)
 	GError *err = NULL;
 	uint64_t n[2], seed;
 
+	if (task->cfg) {
+		rspamd_mime_parser_init_shared(task->cfg);
+	}
+
 	if (RSPAMD_TASK_IS_EMPTY(task)) {
 		/* Don't do anything with empty task */
 		task->flags |= RSPAMD_TASK_FLAG_SKIP_PROCESS;
@@ -1400,7 +1591,7 @@ void rspamd_message_process(struct rspamd_task *task)
 	unsigned int tw, *ptw, dw;
 	struct rspamd_mime_part *part;
 	lua_State *L = NULL;
-	int magic_func_pos = -1, content_func_pos = -1, old_top = -1, funcs_top = -1;
+	int content_func_pos = -1, old_top = -1, funcs_top = -1;
 
 	if (task->cfg) {
 		L = task->cfg->lua_state;
@@ -1408,17 +1599,93 @@ void rspamd_message_process(struct rspamd_task *task)
 
 	rspamd_archives_process(task);
 
+	/* Second pass: fill detected_* for parts not decided during parsing */
+	if (L && task->cfg->mime_parser_cfg &&
+		rspamd_mime_parser_get_lua_magic_cbref(task->cfg->mime_parser_cfg) != -1) {
+		unsigned int j;
+		struct rspamd_mime_part *pp;
+		PTR_ARRAY_FOREACH(MESSAGE_FIELD(task, parts), j, pp)
+		{
+			gboolean needs_refine = FALSE;
+			if (pp->parsed_data.len > 0) {
+				if (pp->detected_type == NULL && pp->detected_ext == NULL) {
+					needs_refine = TRUE;
+				}
+				else if (pp->part_type == RSPAMD_MIME_PART_ARCHIVE) {
+					needs_refine = TRUE;
+				}
+			}
+
+			if (needs_refine) {
+				msg_debug_mime("second-pass lua_magic for part #%ud: reason=%s; ext=%s type=%s",
+							   pp->part_number,
+							   (pp->detected_type == NULL && pp->detected_ext == NULL) ? "undetected" : "archive",
+							   pp->detected_ext ? pp->detected_ext : "(nil)",
+							   pp->detected_type ? pp->detected_type : "(nil)");
+				struct rspamd_mime_part **pmime;
+				struct rspamd_task **ptask;
+				lua_pushcfunction(L, &rspamd_lua_traceback);
+				int err_idx2 = lua_gettop(L);
+				lua_rawgeti(L, LUA_REGISTRYINDEX, rspamd_mime_parser_get_lua_magic_cbref(task->cfg->mime_parser_cfg));
+				pmime = lua_newuserdata(L, sizeof(struct rspamd_mime_part *));
+				rspamd_lua_setclass(L, rspamd_mimepart_classname, -1);
+				*pmime = pp;
+				ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
+				rspamd_lua_setclass(L, rspamd_task_classname, -1);
+				*ptask = task;
+
+				if (lua_pcall(L, 2, 2, err_idx2) == 0) {
+					if (lua_istable(L, -1)) {
+						const char *mb;
+						const char *old_ext = pp->detected_ext;
+						const char *old_type = pp->detected_type;
+						if (lua_isstring(L, -2)) {
+							pp->detected_ext = rspamd_mempool_strdup(task->task_pool, lua_tostring(L, -2));
+						}
+						lua_pushstring(L, "ct");
+						lua_gettable(L, -2);
+						if (lua_isstring(L, -1)) {
+							mb = lua_tostring(L, -1);
+							if (mb) {
+								rspamd_ftok_t srch;
+								srch.begin = mb;
+								srch.len = strlen(mb);
+								pp->detected_ct = rspamd_content_type_parse(srch.begin, srch.len, task->task_pool);
+							}
+						}
+						lua_pop(L, 1);
+						lua_pushstring(L, "type");
+						lua_gettable(L, -2);
+						if (lua_isstring(L, -1)) {
+							pp->detected_type = rspamd_mempool_strdup(task->task_pool, lua_tostring(L, -1));
+						}
+						lua_pop(L, 1);
+						lua_pushstring(L, "no_text");
+						lua_gettable(L, -2);
+						if (lua_isboolean(L, -1) && lua_toboolean(L, -1)) {
+							pp->flags |= RSPAMD_MIME_PART_NO_TEXT_EXTRACTION;
+						}
+						lua_pop(L, 1);
+						msg_debug_mime("second-pass result for part #%ud: ext %s->%s, type %s->%s",
+									   pp->part_number,
+									   old_ext ? old_ext : "(nil)", pp->detected_ext ? pp->detected_ext : "(nil)",
+									   old_type ? old_type : "(nil)", pp->detected_type ? pp->detected_type : "(nil)");
+					}
+				}
+				else {
+					msg_err_task("second-pass detect type: %s", lua_tostring(L, -1));
+				}
+				/* restore stack */
+				lua_settop(L, 0);
+			}
+		}
+	}
+
 	if (L) {
 		old_top = lua_gettop(L);
 	}
 
-	if (L && rspamd_lua_require_function(L,
-										 "lua_magic", "detect_mime_part")) {
-		magic_func_pos = lua_gettop(L);
-	}
-	else {
-		msg_err_task("cannot require lua_magic.detect_mime_part");
-	}
+	/* lua_magic is preloaded by mime parser init; do not require here */
 
 	if (L && rspamd_lua_require_function(L,
 										 "lua_content", "maybe_process_mime_part")) {
@@ -1436,75 +1703,7 @@ void rspamd_message_process(struct rspamd_task *task)
 
 	PTR_ARRAY_FOREACH(MESSAGE_FIELD(task, parts), i, part)
 	{
-		if (magic_func_pos != -1 && part->parsed_data.len > 0) {
-			struct rspamd_mime_part **pmime;
-			struct rspamd_task **ptask;
-
-			lua_pushcfunction(L, &rspamd_lua_traceback);
-			int err_idx = lua_gettop(L);
-			lua_pushvalue(L, magic_func_pos);
-			pmime = lua_newuserdata(L, sizeof(struct rspamd_mime_part *));
-			rspamd_lua_setclass(L, rspamd_mimepart_classname, -1);
-			*pmime = part;
-			ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
-			rspamd_lua_setclass(L, rspamd_task_classname, -1);
-			*ptask = task;
-
-			if (lua_pcall(L, 2, 2, err_idx) != 0) {
-				msg_err_task("cannot detect type: %s", lua_tostring(L, -1));
-			}
-			else {
-				if (lua_istable(L, -1)) {
-					const char *mb;
-
-					/* First returned value */
-					part->detected_ext = rspamd_mempool_strdup(task->task_pool,
-															   lua_tostring(L, -2));
-
-					lua_pushstring(L, "ct");
-					lua_gettable(L, -2);
-
-					if (lua_isstring(L, -1)) {
-						mb = lua_tostring(L, -1);
-
-						if (mb) {
-							rspamd_ftok_t srch;
-
-							srch.begin = mb;
-							srch.len = strlen(mb);
-							part->detected_ct = rspamd_content_type_parse(srch.begin,
-																		  srch.len,
-																		  task->task_pool);
-						}
-					}
-
-					lua_pop(L, 1);
-
-					lua_pushstring(L, "type");
-					lua_gettable(L, -2);
-
-					if (lua_isstring(L, -1)) {
-						part->detected_type = rspamd_mempool_strdup(task->task_pool,
-																	lua_tostring(L, -1));
-					}
-
-					lua_pop(L, 1);
-
-					lua_pushstring(L, "no_text");
-					lua_gettable(L, -2);
-
-					if (lua_isboolean(L, -1)) {
-						if (!!lua_toboolean(L, -1)) {
-							part->flags |= RSPAMD_MIME_PART_NO_TEXT_EXTRACTION;
-						}
-					}
-
-					lua_pop(L, 1);
-				}
-			}
-
-			lua_settop(L, funcs_top);
-		}
+		/* detected_* are already set by mime_parser; no extra lua_magic call here */
 
 		/* Now detect content */
 		if (content_func_pos != -1 && part->parsed_data.len > 0 &&
@@ -1583,7 +1782,7 @@ void rspamd_message_process(struct rspamd_task *task)
 
 		rspamd_mime_part_extract_words(task, text_part);
 
-		if (text_part->utf_words) {
+		if (text_part->utf_words.a) {
 			total_words += text_part->nwords;
 		}
 	}
