@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@
 #include "lua/lua_common.h"
 #include "libserver/cfg_file_private.h"
 #include "libmime/scan_result_private.h"
-#include "contrib/fastutf8/fastutf8.h"
+#include "rspamd_simdutf.h"
 #include <math.h>
 #include "contrib/uthash/utlist.h"
 
@@ -73,7 +73,7 @@ rspamd_metric_actions_foreach_cb(int i, struct rspamd_action *act, void *cbd)
 
 struct rspamd_scan_result *
 rspamd_create_metric_result(struct rspamd_task *task,
-							const gchar *name, gint lua_sym_cbref)
+							const char *name, int lua_sym_cbref)
 {
 	struct rspamd_scan_result *metric_res;
 
@@ -127,10 +127,10 @@ rspamd_pr_sort(const struct rspamd_passthrough_result *pra,
 
 bool rspamd_add_passthrough_result(struct rspamd_task *task,
 								   struct rspamd_action *action,
-								   guint priority,
+								   unsigned int priority,
 								   double target_score,
-								   const gchar *message,
-								   const gchar *module,
+								   const char *message,
+								   const char *module,
 								   uint flags,
 								   struct rspamd_scan_result *scan_result)
 {
@@ -194,23 +194,41 @@ bool rspamd_add_passthrough_result(struct rspamd_task *task,
 	return true;
 }
 
-static inline gdouble
+static inline double
 rspamd_check_group_score(struct rspamd_task *task,
-						 const gchar *symbol,
+						 const char *symbol,
 						 struct rspamd_symbols_group *gr,
-						 gdouble *group_score,
-						 gdouble w)
+						 double *group_score,
+						 double w)
 {
-	if (gr != NULL && group_score && gr->max_score > 0.0 && w > 0.0) {
-		if (*group_score >= gr->max_score && w > 0) {
+	double group_limit = NAN;
+
+	if (gr != NULL && group_score) {
+		if ((*group_score + w) >= 0 && !isnan(gr->max_score) && gr->max_score > 0) {
+			group_limit = gr->max_score;
+		}
+		else if ((*group_score + w) < 0 && !isnan(gr->min_score) && gr->min_score < 0) {
+			group_limit = -gr->min_score;
+		}
+	}
+
+	if (gr != NULL && group_limit && !isnan(group_limit)) {
+		if (fabs(*group_score) >= group_limit && signbit(*group_score) == signbit(w)) {
+			/* Cannot add more to the group */
 			msg_info_task("maximum group score %.2f for group %s has been reached,"
 						  " ignoring symbol %s with weight %.2f",
-						  gr->max_score,
+						  group_limit,
 						  gr->name, symbol, w);
 			return NAN;
 		}
-		else if (*group_score + w > gr->max_score) {
-			w = gr->max_score - *group_score;
+		else if (fabs(*group_score + w) > group_limit) {
+			/* Reduce weight */
+			double new_w = signbit(w) ? -group_limit - *group_score : group_limit - *group_score;
+			msg_info_task("maximum group score %.2f for group %s has been reached,"
+						  " reduce weight of symbol %s from %.2f to %.2f",
+						  group_limit,
+						  gr->name, symbol, w, new_w);
+			w = new_w;
 		}
 	}
 
@@ -223,23 +241,23 @@ rspamd_check_group_score(struct rspamd_task *task,
 
 static struct rspamd_symbol_result *
 insert_metric_result(struct rspamd_task *task,
-					 const gchar *symbol,
+					 const char *symbol,
 					 double weight,
-					 const gchar *opt,
+					 const char *opt,
 					 struct rspamd_scan_result *metric_res,
 					 enum rspamd_symbol_insert_flags flags,
 					 bool *new_sym)
 {
 	struct rspamd_symbol_result *symbol_result = NULL;
-	gdouble final_score, *gr_score = NULL, next_gf = 1.0, diff;
+	double final_score, *gr_score = NULL, diff;
 	struct rspamd_symbol *sdef;
 	struct rspamd_symbols_group *gr = NULL;
 	const ucl_object_t *mobj, *sobj;
-	gint max_shots = G_MAXINT, ret;
-	guint i;
+	int max_shots = G_MAXINT, ret;
+	unsigned int i;
 	khiter_t k;
 	gboolean single = !!(flags & RSPAMD_SYMBOL_INSERT_SINGLE);
-	gchar *sym_cpy;
+	char *sym_cpy;
 
 	if (!isfinite(weight)) {
 		msg_warn_task("detected %s score for symbol %s, replace it with zero",
@@ -290,7 +308,7 @@ insert_metric_result(struct rspamd_task *task,
 	}
 
 	if (task->settings) {
-		gdouble corr;
+		double corr;
 		mobj = ucl_object_lookup(task->settings, "scores");
 
 		if (!mobj) {
@@ -368,22 +386,11 @@ insert_metric_result(struct rspamd_task *task,
 		}
 
 		if (diff) {
-			/* Handle grow factor */
-			if (metric_res->grow_factor && diff > 0) {
-				diff *= metric_res->grow_factor;
-				next_gf *= task->cfg->grow_factor;
-			}
-			else if (diff > 0) {
-				next_gf = task->cfg->grow_factor;
-			}
-
-			msg_debug_metric("adjust grow factor to %.2f for symbol %s (%.2f final)",
-							 next_gf, symbol, diff);
 
 			if (sdef) {
 				PTR_ARRAY_FOREACH(sdef->groups, i, gr)
 				{
-					gdouble cur_diff;
+					double cur_diff;
 
 					k = kh_get(rspamd_symbols_group_hash,
 							   metric_res->sym_groups, gr);
@@ -404,23 +411,13 @@ insert_metric_result(struct rspamd_task *task,
 					}
 					else if (gr_score) {
 						*gr_score += cur_diff;
-
-						if (cur_diff < diff) {
-							/* Reduce */
-							msg_debug_metric(
-								"group limit %.2f is reached for %s when inserting symbol %s;"
-								" reduce score %.2f - %.2f",
-								*gr_score, gr->name, symbol, diff, cur_diff);
-							diff = cur_diff;
-						}
+						diff = cur_diff;
 					}
 				}
 			}
 
 			if (!isnan(diff)) {
 				metric_res->score += diff;
-				metric_res->grow_factor = next_gf;
-
 				if (single) {
 					msg_debug_metric("final score for single symbol %s = %.2f; %.2f diff",
 									 symbol, final_score, diff);
@@ -447,18 +444,6 @@ insert_metric_result(struct rspamd_task *task,
 		symbol_result = rspamd_mempool_alloc0(task->task_pool, sizeof(*symbol_result));
 		kh_value(metric_res->symbols, k) = symbol_result;
 
-		/* Handle grow factor */
-		if (metric_res->grow_factor && final_score > 0) {
-			final_score *= metric_res->grow_factor;
-			next_gf *= task->cfg->grow_factor;
-		}
-		else if (final_score > 0) {
-			next_gf = task->cfg->grow_factor;
-		}
-
-		msg_debug_metric("adjust grow factor to %.2f for symbol %s (%.2f final)",
-						 next_gf, symbol, final_score);
-
 		symbol_result->name = sym_cpy;
 		symbol_result->sym = sdef;
 		symbol_result->nshots = 1;
@@ -467,7 +452,7 @@ insert_metric_result(struct rspamd_task *task,
 			/* Check group limits */
 			PTR_ARRAY_FOREACH(sdef->groups, i, gr)
 			{
-				gdouble cur_score;
+				double cur_score;
 
 				k = kh_get(rspamd_symbols_group_hash, metric_res->sym_groups, gr);
 				g_assert(k != kh_end(metric_res->sym_groups));
@@ -486,15 +471,7 @@ insert_metric_result(struct rspamd_task *task,
 				}
 				else if (gr_score) {
 					*gr_score += cur_score;
-
-					if (cur_score < final_score) {
-						/* Reduce */
-						msg_debug_metric(
-							"group limit %.2f is reached for %s when inserting symbol %s;"
-							" reduce score %.2f - %.2f",
-							*gr_score, gr->name, symbol, final_score, cur_score);
-						final_score = cur_score;
-					}
+					final_score = cur_score;
 				}
 			}
 		}
@@ -503,7 +480,6 @@ insert_metric_result(struct rspamd_task *task,
 			const double epsilon = DBL_EPSILON;
 
 			metric_res->score += final_score;
-			metric_res->grow_factor = next_gf;
 			symbol_result->score = final_score;
 
 			if (final_score > epsilon) {
@@ -535,9 +511,9 @@ insert_metric_result(struct rspamd_task *task,
 
 struct rspamd_symbol_result *
 rspamd_task_insert_result_full(struct rspamd_task *task,
-							   const gchar *symbol,
+							   const char *symbol,
 							   double weight,
-							   const gchar *opt,
+							   const char *opt,
 							   enum rspamd_symbol_insert_flags flags,
 							   struct rspamd_scan_result *result)
 {
@@ -566,7 +542,7 @@ rspamd_task_insert_result_full(struct rspamd_task *task,
 
 				if (!rspamd_lua_universal_pcall(L, mres->symbol_cbref,
 												G_STRLOC, 1, "uss", &err,
-												"rspamd{task}", task, symbol, mres->name ? mres->name : "default")) {
+												rspamd_task_classname, task, symbol, mres->name ? mres->name : "default")) {
 					msg_warn_task("cannot call for symbol_cbref for result %s: %e",
 								  mres->name ? mres->name : "default", err);
 					g_error_free(err);
@@ -638,13 +614,13 @@ rspamd_task_insert_result_full(struct rspamd_task *task,
 	return ret;
 }
 
-static gchar *
+static char *
 rspamd_task_option_safe_copy(struct rspamd_task *task,
-							 const gchar *val,
+							 const char *val,
 							 gsize vlen,
 							 gsize *outlen)
 {
-	const gchar *p, *end;
+	const char *p, *end;
 
 	p = val;
 	end = val + vlen;
@@ -653,7 +629,7 @@ rspamd_task_option_safe_copy(struct rspamd_task *task,
 	while (p < end) {
 		if (*p & 0x80) {
 			UChar32 uc;
-			gint off = 0;
+			int off = 0;
 
 			U8_NEXT(p, off, end - p, uc);
 
@@ -683,7 +659,7 @@ rspamd_task_option_safe_copy(struct rspamd_task *task,
 		}
 	}
 
-	gchar *dest, *d;
+	char *dest, *d;
 
 	dest = rspamd_mempool_alloc(task->task_pool, vlen + 1);
 	d = dest;
@@ -692,7 +668,7 @@ rspamd_task_option_safe_copy(struct rspamd_task *task,
 	while (p < end) {
 		if (*p & 0x80) {
 			UChar32 uc;
-			gint off = 0;
+			int off = 0;
 
 			U8_NEXT(p, off, end - p, uc);
 
@@ -737,15 +713,15 @@ rspamd_task_option_safe_copy(struct rspamd_task *task,
 gboolean
 rspamd_task_add_result_option(struct rspamd_task *task,
 							  struct rspamd_symbol_result *s,
-							  const gchar *val,
+							  const char *val,
 							  gsize vlen)
 {
 	struct rspamd_symbol_option *opt, srch;
 	gboolean ret = FALSE;
-	gchar *opt_cpy = NULL;
+	char *opt_cpy = NULL;
 	gsize cpy_len;
 	khiter_t k;
-	gint r;
+	int r;
 	struct rspamd_symbol_result *cur;
 
 	if (s && val) {
@@ -779,14 +755,14 @@ rspamd_task_add_result_option(struct rspamd_task *task,
 
 			if (!(cur->sym && (cur->sym->flags & RSPAMD_SYMBOL_FLAG_ONEPARAM))) {
 
-				srch.option = (gchar *) val;
+				srch.option = (char *) val;
 				srch.optlen = vlen;
 				k = kh_get(rspamd_options_hash, cur->options, &srch);
 
 				if (k == kh_end(cur->options)) {
 					opt_cpy = rspamd_task_option_safe_copy(task, val, vlen, &cpy_len);
 					if (cpy_len != vlen) {
-						srch.option = (gchar *) opt_cpy;
+						srch.option = (char *) opt_cpy;
 						srch.optlen = cpy_len;
 						k = kh_get(rspamd_options_hash, cur->options, &srch);
 					}
@@ -1013,7 +989,7 @@ rspamd_task_find_symbol_result(struct rspamd_task *task, const char *sym,
 
 struct rspamd_symbol_result *rspamd_task_remove_symbol_result(
 	struct rspamd_task *task,
-	const gchar *symbol,
+	const char *symbol,
 	struct rspamd_scan_result *result)
 {
 	struct rspamd_symbol_result *res = NULL;
@@ -1036,12 +1012,12 @@ struct rspamd_symbol_result *rspamd_task_remove_symbol_result(
 			/* Also check the group limit */
 			if (result->sym_groups && res->sym) {
 				struct rspamd_symbol_group *gr;
-				gint i;
+				int i;
 				khiter_t k_groups;
 
 				PTR_ARRAY_FOREACH(res->sym->groups, i, gr)
 				{
-					gdouble *gr_score;
+					double *gr_score;
 
 					k_groups = kh_get(rspamd_symbols_group_hash,
 									  result->sym_groups, gr);
@@ -1070,7 +1046,7 @@ void rspamd_task_symbol_result_foreach(struct rspamd_task *task,
 									   struct rspamd_scan_result *result, GHFunc func,
 									   gpointer ud)
 {
-	const gchar *kk;
+	const char *kk;
 	struct rspamd_symbol_result *res;
 
 	if (result == NULL) {
@@ -1087,7 +1063,7 @@ void rspamd_task_symbol_result_foreach(struct rspamd_task *task,
 
 struct rspamd_scan_result *
 rspamd_find_metric_result(struct rspamd_task *task,
-						  const gchar *name)
+						  const char *name)
 {
 	struct rspamd_scan_result *res;
 
@@ -1104,3 +1080,49 @@ rspamd_find_metric_result(struct rspamd_task *task,
 
 	return NULL;
 }
+
+void rspamd_task_result_adjust_grow_factor(struct rspamd_task *task,
+										   struct rspamd_scan_result *result,
+										   double grow_factor)
+{
+	const char *kk;
+	struct rspamd_symbol_result *res;
+	double final_grow_factor = grow_factor;
+	double max_limit = G_MINDOUBLE;
+
+	if (grow_factor > 1.0) {
+
+		for (unsigned int i = 0; i < result->nactions; i++) {
+			struct rspamd_action_config *cur = &result->actions_config[i];
+
+			if (cur->cur_limit > 0 && max_limit < cur->cur_limit) {
+				max_limit = cur->cur_limit;
+			}
+		}
+
+		/* Adjust factor by selecting all symbols and checking those with positive scores */
+		kh_foreach(result->symbols, kk, res, {
+			if (res->score > 0) {
+				double mult = grow_factor - 1.0;
+				/* We adjust the factor by the ratio of the score to the max limit */
+				if (max_limit > 0 && !isnan(res->score)) {
+					mult *= res->score / max_limit;
+					final_grow_factor *= 1.0 + mult;
+				}
+			}
+		});
+
+		/* At this stage we know that we have some grow factor to apply */
+		if (final_grow_factor > 1.0) {
+			msg_info_task("calculated final grow factor for task: %.3f (%.2f the original one)",
+						  final_grow_factor, grow_factor);
+			kh_foreach(result->symbols, kk, res, {
+				if (res->score > 0) {
+					result->score -= res->score;
+					res->score *= final_grow_factor;
+					result->score += res->score;
+				}
+			});
+		}
+	}
+}
