@@ -48,6 +48,8 @@ struct worker_s;
 struct rspamd_external_libs_ctx;
 struct rspamd_cryptobox_pubkey;
 struct rspamd_dns_resolver;
+struct rspamd_tokenizer_manager;
+struct rspamd_mime_parser_config;
 
 /**
  * Logging type
@@ -138,7 +140,10 @@ struct rspamd_statfile_config {
 	char *symbol;                          /**< symbol of statfile									*/
 	char *label;                           /**< label of this statfile								*/
 	ucl_object_t *opts;                    /**< other options										*/
-	gboolean is_spam;                      /**< spam flag											*/
+	char *class_name;                      /**< class name for multi-class classification			*/
+	unsigned int class_index;              /**< class index for O(1) lookup during classification	*/
+	gboolean is_spam;                      /**< DEPRECATED: spam flag - use class_name instead		*/
+	gboolean is_spam_converted;            /**< TRUE if class_name was converted from is_spam flag	*/
 	struct rspamd_classifier_config *clcf; /**< parent pointer of classifier configuration			*/
 	gpointer data;                         /**< opaque data 										*/
 };
@@ -181,6 +186,8 @@ struct rspamd_classifier_config {
 	double min_prob_strength;                  /**< use only tokens with probability in [0.5 - MPS, 0.5 + MPS] */
 	unsigned int min_learns;                   /**< minimum number of learns for each statfile			*/
 	unsigned int flags;
+	GHashTable *class_labels; /**< class_name -> backend_symbol mapping for multi-class */
+	GPtrArray *class_names;   /**< ordered list of class names						*/
 };
 
 struct rspamd_worker_bind_conf {
@@ -455,6 +462,8 @@ struct rspamd_config {
 	double upstream_revive_time;              /**< revive timeout for upstreams						*/
 	double upstream_lazy_resolve_time;        /**< lazy resolve time for upstreams					*/
 	double upstream_resolve_min_interval;     /**< minimum interval for resolving attempts (60 seconds by default) */
+	double upstream_probe_max_backoff;        /**< maximum backoff for probe retries when all upstreams are down */
+	double upstream_probe_jitter;             /**< jitter coefficient applied to probe backoff scheduling */
 	struct upstream_ctx *ups_ctx;             /**< upstream context									*/
 	struct rspamd_dns_resolver *dns_resolver; /**< dns resolver if loaded								*/
 
@@ -482,7 +491,8 @@ struct rspamd_config {
 	struct rspamd_monitored_ctx *monitored_ctx; /**< context for monitored resources					*/
 	void *redis_pool;                           /**< redis connection pool								*/
 
-	struct rspamd_re_cache *re_cache; /**< static regexp cache								*/
+	struct rspamd_re_cache *re_cache;                  /**< static regexp cache								*/
+	struct rspamd_mime_parser_config *mime_parser_cfg; /**< mime parser shared config */
 
 	GHashTable *trusted_keys; /**< list of trusted public keys						*/
 
@@ -497,9 +507,10 @@ struct rspamd_config {
 	char *zstd_output_dictionary; /**< path to zstd output dictionary						*/
 	ucl_object_t *neighbours;     /**< other servers in the cluster						*/
 
-	struct rspamd_config_settings_elt *setting_ids; /**< preprocessed settings ids							*/
-	struct rspamd_lang_detector *lang_det;          /**< language detector									*/
-	struct rspamd_worker *cur_worker;               /**< set dynamically by each worker							*/
+	struct rspamd_config_settings_elt *setting_ids;     /**< preprocessed settings ids							*/
+	struct rspamd_lang_detector *lang_det;              /**< language detector									*/
+	struct rspamd_tokenizer_manager *tokenizer_manager; /**< custom tokenizer manager						*/
+	struct rspamd_worker *cur_worker;                   /**< set dynamically by each worker							*/
 
 	ref_entry_t ref; /**< reference counter									*/
 };
@@ -619,12 +630,25 @@ void rspamd_config_insert_classify_symbols(struct rspamd_config *cfg);
  */
 gboolean rspamd_config_check_statfiles(struct rspamd_classifier_config *cf);
 
-/*
- * Find classifier config by name
+/**
+ * Multi-class configuration helpers
+ */
+gboolean rspamd_config_parse_class_labels(const ucl_object_t *obj,
+										  GHashTable **class_labels);
+
+gboolean rspamd_config_migrate_binary_config(struct rspamd_statfile_config *stcf);
+
+gboolean rspamd_config_validate_class_config(struct rspamd_classifier_config *ccf,
+											 GError **err);
+
+const char *rspamd_config_get_class_label(struct rspamd_classifier_config *ccf,
+										  const char *class_name);
+
+/**
+ * Find classifier by name
  */
 struct rspamd_classifier_config *rspamd_config_find_classifier(
-	struct rspamd_config *cfg,
-	const char *name);
+	struct rspamd_config *cfg, const char *name);
 
 void rspamd_ucl_add_conf_macros(struct ucl_parser *parser,
 								struct rspamd_config *cfg);
