@@ -20,6 +20,8 @@
 #include "contrib/libev/ev.h"
 #include "dns.h"
 #include "ref.h"
+#include <stdbool.h>
+#include <stdint.h>
 
 
 /* Main types and definitions */
@@ -118,15 +120,15 @@ struct rspamd_dkim_check_result {
 	enum rspamd_dkim_check_rcode rcode;
 	rspamd_dkim_context_t *ctx;
 	/* Processed parts */
-	const gchar *selector;
-	const gchar *domain;
-	const gchar *short_b;
-	const gchar *fail_reason;
+	const char *selector;
+	const char *domain;
+	const char *short_b;
+	const char *fail_reason;
 };
 
 
 /* Err MUST be freed if it is not NULL, key is allocated by slice allocator */
-typedef void (*dkim_key_handler_f)(rspamd_dkim_key_t *key, gsize keylen,
+typedef void (*dkim_key_handler_f)(rspamd_dkim_key_t *key, size_t keylen,
 								   rspamd_dkim_context_t *ctx, gpointer ud, GError *err);
 
 /**
@@ -137,10 +139,10 @@ typedef void (*dkim_key_handler_f)(rspamd_dkim_key_t *key, gsize keylen,
  * @param err pointer to error object
  * @return new context or NULL
  */
-rspamd_dkim_context_t *rspamd_create_dkim_context(const gchar *sig,
+rspamd_dkim_context_t *rspamd_create_dkim_context(const char *sig,
 												  rspamd_mempool_t *pool,
 												  struct rspamd_dns_resolver *resolver,
-												  guint time_jitter,
+												  unsigned int time_jitter,
 												  enum rspamd_dkim_type type,
 												  GError **err);
 
@@ -153,9 +155,9 @@ rspamd_dkim_context_t *rspamd_create_dkim_context(const gchar *sig,
  */
 rspamd_dkim_sign_context_t *rspamd_create_dkim_sign_context(struct rspamd_task *task,
 															rspamd_dkim_sign_key_t *priv_key,
-															gint headers_canon,
-															gint body_canon,
-															const gchar *dkim_headers,
+															int headers_canon,
+															int body_canon,
+															const char *dkim_headers,
 															enum rspamd_dkim_type type,
 															GError **err);
 
@@ -165,7 +167,7 @@ rspamd_dkim_sign_context_t *rspamd_create_dkim_sign_context(struct rspamd_task *
  * @param err
  * @return
  */
-rspamd_dkim_sign_key_t *rspamd_dkim_sign_key_load(const gchar *what, gsize len,
+rspamd_dkim_sign_key_t *rspamd_dkim_sign_key_load(const char *what, size_t len,
 												  enum rspamd_dkim_key_format type,
 												  GError **err);
 
@@ -174,8 +176,8 @@ rspamd_dkim_sign_key_t *rspamd_dkim_sign_key_load(const gchar *what, gsize len,
  * @param key
  * @return
 */
-gboolean rspamd_dkim_sign_key_maybe_invalidate(rspamd_dkim_sign_key_t *key,
-											   time_t mtime);
+bool rspamd_dkim_sign_key_maybe_invalidate(rspamd_dkim_sign_key_t *key,
+										   time_t mtime);
 
 /**
  * Make DNS request for specified context and obtain and parse key
@@ -184,10 +186,10 @@ gboolean rspamd_dkim_sign_key_maybe_invalidate(rspamd_dkim_sign_key_t *key,
  * @param s async session to make request
  * @return
  */
-gboolean rspamd_get_dkim_key(rspamd_dkim_context_t *ctx,
-							 struct rspamd_task *task,
-							 dkim_key_handler_f handler,
-							 gpointer ud);
+bool rspamd_get_dkim_key(rspamd_dkim_context_t *ctx,
+						 struct rspamd_task *task,
+						 dkim_key_handler_f handler,
+						 gpointer ud);
 
 /**
  * Check task for dkim context using dkim key
@@ -206,12 +208,12 @@ rspamd_dkim_create_result(rspamd_dkim_context_t *ctx,
 						  struct rspamd_task *task);
 
 GString *rspamd_dkim_sign(struct rspamd_task *task,
-						  const gchar *selector,
-						  const gchar *domain,
+						  const char *selector,
+						  const char *domain,
 						  time_t expire,
-						  gsize len,
-						  guint idx,
-						  const gchar *arc_cv,
+						  size_t len,
+						  unsigned int idx,
+						  const char *arc_cv,
 						  rspamd_dkim_sign_context_t *ctx);
 
 rspamd_dkim_key_t *rspamd_dkim_key_ref(rspamd_dkim_key_t *k);
@@ -222,13 +224,13 @@ rspamd_dkim_sign_key_t *rspamd_dkim_sign_key_ref(rspamd_dkim_sign_key_t *k);
 
 void rspamd_dkim_sign_key_unref(rspamd_dkim_sign_key_t *k);
 
-const gchar *rspamd_dkim_get_domain(rspamd_dkim_context_t *ctx);
+const char *rspamd_dkim_get_domain(rspamd_dkim_context_t *ctx);
 
-const gchar *rspamd_dkim_get_selector(rspamd_dkim_context_t *ctx);
+const char *rspamd_dkim_get_selector(rspamd_dkim_context_t *ctx);
 
-const gchar *rspamd_dkim_get_dns_key(rspamd_dkim_context_t *ctx);
+const char *rspamd_dkim_get_dns_key(rspamd_dkim_context_t *ctx);
 
-guint rspamd_dkim_key_get_ttl(rspamd_dkim_key_t *k);
+unsigned int rspamd_dkim_key_get_ttl(rspamd_dkim_key_t *k);
 
 /**
  * Create DKIM public key from a raw data
@@ -238,7 +240,7 @@ guint rspamd_dkim_key_get_ttl(rspamd_dkim_key_t *k);
  * @param err
  * @return
  */
-rspamd_dkim_key_t *rspamd_dkim_make_key(const gchar *keydata, guint keylen,
+rspamd_dkim_key_t *rspamd_dkim_make_key(const char *keydata, unsigned int keylen,
 										enum rspamd_dkim_key_type type,
 										GError **err);
 
@@ -249,7 +251,7 @@ rspamd_dkim_key_t *rspamd_dkim_make_key(const gchar *keydata, guint keylen,
  * @param key
  * @return
  */
-const guchar *rspamd_dkim_key_id(rspamd_dkim_key_t *key);
+const unsigned char *rspamd_dkim_key_id(rspamd_dkim_key_t *key);
 
 /**
  * Parse DKIM public key from a TXT record
@@ -258,7 +260,7 @@ const guchar *rspamd_dkim_key_id(rspamd_dkim_key_t *key);
  * @param err
  * @return
  */
-rspamd_dkim_key_t *rspamd_dkim_parse_key(const gchar *txt, gsize *keylen,
+rspamd_dkim_key_t *rspamd_dkim_parse_key(const char *txt, size_t *keylen,
 										 GError **err);
 
 /**
@@ -269,10 +271,10 @@ rspamd_dkim_key_t *rspamd_dkim_parse_key(const gchar *txt, gsize *keylen,
  * @param outlen
  * @return
  */
-goffset rspamd_dkim_canonize_header_relaxed_str(const gchar *hname,
-												const gchar *hvalue,
-												gchar *out,
-												gsize outlen);
+off_t rspamd_dkim_canonize_header_relaxed_str(const char *hname,
+											  const char *hvalue,
+											  char *out,
+											  size_t outlen);
 
 /**
  * Checks public and private keys for match
@@ -281,9 +283,9 @@ goffset rspamd_dkim_canonize_header_relaxed_str(const gchar *hname,
  * @param err
  * @return
  */
-gboolean rspamd_dkim_match_keys(rspamd_dkim_key_t *pk,
-								rspamd_dkim_sign_key_t *sk,
-								GError **err);
+bool rspamd_dkim_match_keys(rspamd_dkim_key_t *pk,
+							rspamd_dkim_sign_key_t *sk,
+							GError **err);
 
 /**
  * Free DKIM key
