@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include "cfg_file.h"
 #include "contrib/hiredis/hiredis.h"
 #include "contrib/hiredis/async.h"
+#include "contrib/hiredis/hiredis_ssl.h"
 #include "contrib/hiredis/adapters/libev.h"
 #include "cryptobox.h"
 #include "logger.h"
@@ -53,7 +54,7 @@ struct redis_pool_connection {
 	redis_pool *pool;
 	conn_iter_t elt_pos;
 	ev_timer timeout;
-	gchar tag[MEMPOOL_UID_LEN];
+	char tag[MEMPOOL_UID_LEN];
 	rspamd_redis_pool_connection_state state;
 
 	auto schedule_timeout() -> void;
@@ -90,6 +91,14 @@ class redis_pool_elt {
 	std::string db;
 	std::string username;
 	std::string password;
+	/* TLS options */
+	bool use_tls = false;
+	bool no_ssl_verify = false;
+	std::string ca_file;
+	std::string ca_dir;
+	std::string cert_file;
+	std::string key_file;
+	std::string sni;
 	int port;
 	redis_pool_key_t key;
 	bool is_unix;
@@ -102,11 +111,20 @@ public:
 	redis_pool_elt(redis_pool_elt &&other) = default;
 
 	explicit redis_pool_elt(redis_pool *_pool,
-							const gchar *_db, const gchar *_username,
-							const gchar *_password,
-							const char *_ip, int _port)
+							const char *_db, const char *_username,
+							const char *_password,
+							const char *_ip, int _port,
+							bool _use_tls = false,
+							bool _no_ssl_verify = false,
+							const char *_ca_file = nullptr,
+							const char *_ca_dir = nullptr,
+							const char *_cert_file = nullptr,
+							const char *_key_file = nullptr,
+							const char *_sni = nullptr)
 		: pool(_pool), ip(_ip), port(_port),
-		  key(redis_pool_elt::make_key(_db, _username, _password, _ip, _port))
+		  key(redis_pool_elt::make_key(_db, _username, _password, _ip, _port,
+									   _use_tls, _no_ssl_verify,
+									   _ca_file, _ca_dir, _cert_file, _key_file, _sni))
 	{
 		is_unix = ip[0] == '.' || ip[0] == '/';
 
@@ -119,6 +137,14 @@ public:
 		if (_password) {
 			password = _password;
 		}
+
+		use_tls = _use_tls;
+		no_ssl_verify = _no_ssl_verify;
+		if (_ca_file) ca_file = _ca_file;
+		if (_ca_dir) ca_dir = _ca_dir;
+		if (_cert_file) cert_file = _cert_file;
+		if (_key_file) key_file = _key_file;
+		if (_sni) sni = _sni;
 	}
 
 	auto new_connection() -> redisAsyncContext *;
@@ -150,8 +176,12 @@ public:
 		conn->elt_pos = std::prev(std::end(terminating));
 	}
 
-	inline static auto make_key(const gchar *db, const gchar *username,
-								const gchar *password, const char *ip, int port) -> redis_pool_key_t
+	inline static auto make_key(const char *db, const char *username,
+								const char *password, const char *ip, int port,
+								bool use_tls = false, bool no_ssl_verify = false,
+								const char *ca_file = nullptr, const char *ca_dir = nullptr,
+								const char *cert_file = nullptr, const char *key_file = nullptr,
+								const char *sni = nullptr) -> redis_pool_key_t
 	{
 		rspamd_cryptobox_fast_hash_state_t st;
 
@@ -170,6 +200,25 @@ public:
 		rspamd_cryptobox_fast_hash_update(&st, ip, strlen(ip));
 		rspamd_cryptobox_fast_hash_update(&st, &port, sizeof(port));
 
+		/* TLS parameters */
+		rspamd_cryptobox_fast_hash_update(&st, &use_tls, sizeof(use_tls));
+		rspamd_cryptobox_fast_hash_update(&st, &no_ssl_verify, sizeof(no_ssl_verify));
+		if (ca_file) {
+			rspamd_cryptobox_fast_hash_update(&st, ca_file, strlen(ca_file));
+		}
+		if (ca_dir) {
+			rspamd_cryptobox_fast_hash_update(&st, ca_dir, strlen(ca_dir));
+		}
+		if (cert_file) {
+			rspamd_cryptobox_fast_hash_update(&st, cert_file, strlen(cert_file));
+		}
+		if (key_file) {
+			rspamd_cryptobox_fast_hash_update(&st, key_file, strlen(key_file));
+		}
+		if (sni) {
+			rspamd_cryptobox_fast_hash_update(&st, sni, strlen(sni));
+		}
+
 		return rspamd_cryptobox_fast_hash_final(&st);
 	}
 
@@ -184,6 +233,26 @@ public:
 	}
 
 private:
+	auto initiate_tls(redisAsyncContext *ctx) const -> bool
+	{
+		redisSSLContextError ssl_err = REDIS_SSL_CTX_NONE;
+		redisSSLOptions opts{};
+		opts.cacert_filename = ca_file.empty() ? nullptr : ca_file.c_str();
+		opts.capath = ca_dir.empty() ? nullptr : ca_dir.c_str();
+		opts.cert_filename = cert_file.empty() ? nullptr : cert_file.c_str();
+		opts.private_key_filename = key_file.empty() ? nullptr : key_file.c_str();
+		opts.server_name = sni.empty() ? nullptr : sni.c_str();
+		opts.verify_mode = no_ssl_verify ? REDIS_SSL_VERIFY_NONE : REDIS_SSL_VERIFY_PEER;
+		auto *ssl_ctx = redisCreateSSLContextWithOptions(&opts, &ssl_err);
+		if (!ssl_ctx || redisInitiateSSLWithContext(&ctx->c, ssl_ctx) != REDIS_OK) {
+			msg_err("cannot start TLS for redis %s:%d: %s", ip.c_str(), port, ctx->errstr);
+			if (ssl_ctx) redisFreeSSLContext(ssl_ctx);
+			return false;
+		}
+		redisFreeSSLContext(ssl_ctx);
+		return true;
+	}
+
 	auto redis_async_new() -> redisAsyncContext *
 	{
 		struct redisAsyncContext *ctx;
@@ -241,8 +310,15 @@ public:
 		cfg = _cfg;
 	}
 
-	auto new_connection(const gchar *db, const gchar *username,
-						const gchar *password, const char *ip, int port) -> redisAsyncContext *;
+	auto new_connection(const char *db, const char *username,
+						const char *password, const char *ip, int port) -> redisAsyncContext *;
+
+	auto new_connection_ext(const char *db, const char *username,
+							const char *password, const char *ip, int port,
+							bool use_tls, bool no_ssl_verify,
+							const char *ca_file, const char *ca_dir,
+							const char *cert_file, const char *key_file,
+							const char *sni) -> redisAsyncContext *;
 
 	auto release_connection(redisAsyncContext *ctx,
 							enum rspamd_redis_pool_release_type how) -> void;
@@ -442,6 +518,8 @@ redis_pool_connection::redis_pool_connection(redis_pool *_pool,
 	}
 }
 
+/* OpenSSL is initialised by core before Redis pool is used */
+
 auto redis_pool_elt::new_connection() -> redisAsyncContext *
 {
 	if (!inactive.empty()) {
@@ -452,8 +530,8 @@ auto redis_pool_elt::new_connection() -> redisAsyncContext *
 		g_assert(conn->state != rspamd_redis_pool_connection_state::RSPAMD_REDIS_POOL_CONN_ACTIVE);
 		if (conn->ctx->err == REDIS_OK) {
 			/* Also check SO_ERROR */
-			gint err;
-			socklen_t len = sizeof(gint);
+			int err;
+			socklen_t len = sizeof(int);
 
 			if (getsockopt(conn->ctx->c.fd, SOL_SOCKET, SO_ERROR,
 						   (void *) &err, &len) == -1) {
@@ -465,6 +543,8 @@ auto redis_pool_elt::new_connection() -> redisAsyncContext *
 				 * We cannot reuse connection, so we just recursively call
 				 * this function one more time
 				 */
+				msg_debug_rpool("cannot reuse the existing connection to %s:%d: %p; errno=%d",
+								ip.c_str(), port, conn->ctx, err);
 				return new_connection();
 			}
 			else {
@@ -481,10 +561,23 @@ auto redis_pool_elt::new_connection() -> redisAsyncContext *
 		}
 		else {
 			auto *nctx = redis_async_new();
+			msg_debug_rpool("error in the inactive connection: %s; opened new connection to %s:%d: %p",
+							conn->ctx->errstr, ip.c_str(), port, nctx);
+
 			if (nctx) {
-				active.emplace_front(std::make_unique<redis_pool_connection>(pool, this,
-																			 db.c_str(), username.c_str(), password.c_str(), nctx));
-				active.front()->elt_pos = active.begin();
+				/* If TLS is configured for this element, initiate it now */
+				if (use_tls && !is_unix) {
+					if (!initiate_tls(nctx)) {
+						redisAsyncFree(nctx);
+						nctx = nullptr;
+					}
+				}
+
+				if (nctx) {
+					active.emplace_front(std::make_unique<redis_pool_connection>(pool, this,
+																				 db.c_str(), username.c_str(), password.c_str(), nctx));
+					active.front()->elt_pos = active.begin();
+				}
 			}
 
 			return nctx;
@@ -492,10 +585,24 @@ auto redis_pool_elt::new_connection() -> redisAsyncContext *
 	}
 	else {
 		auto *nctx = redis_async_new();
+
 		if (nctx) {
-			active.emplace_front(std::make_unique<redis_pool_connection>(pool, this,
-																		 db.c_str(), username.c_str(), password.c_str(), nctx));
-			active.front()->elt_pos = active.begin();
+			/* If TLS is configured for this element, initiate it now */
+			if (use_tls && !is_unix) {
+				if (!initiate_tls(nctx)) {
+					redisAsyncFree(nctx);
+					nctx = nullptr;
+				}
+			}
+
+			if (nctx) {
+				active.emplace_front(std::make_unique<redis_pool_connection>(pool, this,
+																			 db.c_str(), username.c_str(), password.c_str(), nctx));
+				active.front()->elt_pos = active.begin();
+				auto conn = active.front().get();
+				msg_debug_rpool("no inactive connections; opened new connection to %s:%d: %p",
+								ip.c_str(), port, nctx);
+			}
 		}
 
 		return nctx;
@@ -504,8 +611,8 @@ auto redis_pool_elt::new_connection() -> redisAsyncContext *
 	RSPAMD_UNREACHABLE;
 }
 
-auto redis_pool::new_connection(const gchar *db, const gchar *username,
-								const gchar *password, const char *ip, int port) -> redisAsyncContext *
+auto redis_pool::new_connection(const char *db, const char *username,
+								const char *password, const char *ip, int port) -> redisAsyncContext *
 {
 
 	if (!wanna_die) {
@@ -529,6 +636,39 @@ auto redis_pool::new_connection(const gchar *db, const gchar *username,
 	return nullptr;
 }
 
+auto redis_pool::new_connection_ext(const char *db, const char *username,
+									const char *password, const char *ip, int port,
+									bool use_tls, bool no_ssl_verify,
+									const char *ca_file, const char *ca_dir,
+									const char *cert_file, const char *key_file,
+									const char *sni) -> redisAsyncContext *
+{
+
+	if (!wanna_die) {
+		auto key = redis_pool_elt::make_key(db, username, password, ip, port,
+											use_tls, no_ssl_verify, ca_file, ca_dir,
+											cert_file, key_file, sni);
+		auto found_elt = elts_by_key.find(key);
+
+		if (found_elt != elts_by_key.end()) {
+			auto &elt = found_elt->second;
+
+			return elt.new_connection();
+		}
+		else {
+			/* Need to create a pool */
+			auto nelt = elts_by_key.try_emplace(key,
+												this, db, username, password, ip, port,
+												use_tls, no_ssl_verify, ca_file, ca_dir,
+												cert_file, key_file, sni);
+
+			return nelt.first->second.new_connection();
+		}
+	}
+
+	return nullptr;
+}
+
 auto redis_pool::release_connection(redisAsyncContext *ctx,
 									enum rspamd_redis_pool_release_type how) -> void
 {
@@ -602,8 +742,8 @@ void rspamd_redis_pool_config(void *p,
 
 struct redisAsyncContext *
 rspamd_redis_pool_connect(void *p,
-						  const gchar *db, const gchar *username,
-						  const gchar *password, const char *ip, int port)
+						  const char *db, const char *username,
+						  const char *password, const char *ip, int port)
 {
 	g_assert(p != NULL);
 	auto *pool = reinterpret_cast<class rspamd::redis_pool *>(p);
@@ -612,6 +752,28 @@ rspamd_redis_pool_connect(void *p,
 }
 
 
+struct redisAsyncContext *
+rspamd_redis_pool_connect_ext(void *p,
+							  const char *db, const char *username,
+							  const char *password, const char *ip, int port,
+							  const struct rspamd_redis_tls_opts *tls)
+{
+	g_assert(p != NULL);
+	auto *pool = reinterpret_cast<class rspamd::redis_pool *>(p);
+
+	if (tls && tls->use_tls) {
+		return pool->new_connection_ext(db, username, password, ip, port,
+										true, tls->no_ssl_verify,
+										tls->ca_file, tls->ca_dir,
+										tls->cert_file, tls->key_file,
+										tls->sni);
+	}
+	else {
+		return pool->new_connection(db, username, password, ip, port);
+	}
+}
+
+
 void rspamd_redis_pool_release_connection(void *p,
 										  struct redisAsyncContext *ctx, enum rspamd_redis_pool_release_type how)
 {
@@ -631,10 +793,10 @@ void rspamd_redis_pool_destroy(void *p)
 	delete pool;
 }
 
-const gchar *
+const char *
 rspamd_redis_type_to_string(int type)
 {
-	const gchar *ret = "unknown";
+	const char *ret = "unknown";
 
 	switch (type) {
 	case REDIS_REPLY_STRING:
