@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,7 +15,7 @@
  */
 #include "fstring.h"
 #include "str_util.h"
-#include "contrib/fastutf8/fastutf8.h"
+#include "rspamd_simdutf.h"
 #include "contrib/mumhash/mum.h"
 
 
@@ -72,7 +72,7 @@ rspamd_fstring_sized_new(gsize initial_size)
 }
 
 rspamd_fstring_t *
-rspamd_fstring_new_init(const gchar *init, gsize len)
+rspamd_fstring_new_init(const char *init, gsize len)
 {
 	rspamd_fstring_t *s;
 	gsize real_size = MAX(default_initial_size, len);
@@ -92,7 +92,7 @@ rspamd_fstring_new_init(const gchar *init, gsize len)
 }
 
 rspamd_fstring_t *
-rspamd_fstring_assign(rspamd_fstring_t *str, const gchar *init, gsize len)
+rspamd_fstring_assign(rspamd_fstring_t *str, const char *init, gsize len)
 {
 	gsize avail;
 
@@ -229,8 +229,8 @@ void rspamd_fstring_erase(rspamd_fstring_t *str, gsize pos, gsize len)
 }
 
 /* Compat code */
-static guint64
-fstrhash_c(guint64 c, guint64 hval)
+static uint64_t
+fstrhash_c(uint64_t c, uint64_t hval)
 {
 	return mum_hash_step(hval, c);
 }
@@ -239,13 +239,11 @@ fstrhash_c(guint64 c, guint64 hval)
 /*
  * Return hash value for a string
  */
-guint32
+uint32_t
 rspamd_fstrhash_lc(const rspamd_ftok_t *str, gboolean is_utf)
 {
-	gsize i;
-	guint64 hval;
-	const gchar *p, *end = NULL;
-	gunichar uc;
+	uint64_t hval;
+	const char *p;
 
 	if (str == NULL) {
 		return 0;
@@ -253,34 +251,39 @@ rspamd_fstrhash_lc(const rspamd_ftok_t *str, gboolean is_utf)
 
 	p = str->begin;
 	hval = str->len;
-	end = p + str->len;
 
 	if (is_utf) {
 		if (rspamd_fast_utf8_validate(p, str->len) != 0) {
 			return rspamd_fstrhash_lc(str, FALSE);
 		}
-		while (p < end) {
-			uc = g_unichar_tolower(g_utf8_get_char(p));
-			hval = fstrhash_c(uc, hval);
-			p = g_utf8_next_char(p);
+
+		size_t i = 0, len = str->len;
+		UChar32 uc;
+
+		while (i < len) {
+			U8_NEXT(p, i, len, uc);
+
+			if (uc > 0) {
+				hval = fstrhash_c(u_tolower(uc), hval);
+			}
 		}
 	}
 	else {
-		gsize large_steps = str->len / sizeof(guint64);
-		for (i = 0; i < large_steps; i++, p += sizeof(guint64)) {
+		gsize large_steps = str->len / sizeof(uint64_t);
+		for (size_t i = 0; i < large_steps; i++, p += sizeof(uint64_t)) {
 			/* Copy to the uint64 lowercasing each byte */
 			union {
-				char c[sizeof(guint64)];
-				guint64 iu64;
+				char c[sizeof(uint64_t)];
+				uint64_t iu64;
 			} t;
-			for (int j = 0; j < sizeof(guint64); j++) {
+			for (int j = 0; j < sizeof(uint64_t); j++) {
 				t.c[j] = g_ascii_tolower(p[j]);
 			}
 			hval = fstrhash_c(t.iu64, hval);
 		}
 
-		gsize remain = str->len % sizeof(guint64);
-		for (i = 0; i < remain; i++, p++) {
+		gsize remain = str->len % sizeof(uint64_t);
+		for (size_t i = 0; i < remain; i++, p++) {
 			hval = fstrhash_c(g_ascii_tolower(*p), hval);
 		}
 	}
@@ -301,10 +304,10 @@ rspamd_fstring_equal(const rspamd_fstring_t *s1,
 	return FALSE;
 }
 
-gint rspamd_fstring_casecmp(const rspamd_fstring_t *s1,
-							const rspamd_fstring_t *s2)
+int rspamd_fstring_casecmp(const rspamd_fstring_t *s1,
+						   const rspamd_fstring_t *s2)
 {
-	gint ret = 0;
+	int ret = 0;
 
 	g_assert(s1 != NULL && s2 != NULL);
 
@@ -318,8 +321,8 @@ gint rspamd_fstring_casecmp(const rspamd_fstring_t *s1,
 	return ret;
 }
 
-gint rspamd_fstring_cmp(const rspamd_fstring_t *s1,
-						const rspamd_fstring_t *s2)
+int rspamd_fstring_cmp(const rspamd_fstring_t *s1,
+					   const rspamd_fstring_t *s2)
 {
 	g_assert(s1 != NULL && s2 != NULL);
 
@@ -330,10 +333,10 @@ gint rspamd_fstring_cmp(const rspamd_fstring_t *s1,
 	return s1->len - s2->len;
 }
 
-gint rspamd_ftok_casecmp(const rspamd_ftok_t *s1,
-						 const rspamd_ftok_t *s2)
+int rspamd_ftok_casecmp(const rspamd_ftok_t *s1,
+						const rspamd_ftok_t *s2)
 {
-	gint ret = 0;
+	int ret = 0;
 
 	g_assert(s1 != NULL && s2 != NULL);
 
@@ -347,8 +350,8 @@ gint rspamd_ftok_casecmp(const rspamd_ftok_t *s1,
 	return ret;
 }
 
-gint rspamd_ftok_cmp(const rspamd_ftok_t *s1,
-					 const rspamd_ftok_t *s2)
+int rspamd_ftok_cmp(const rspamd_ftok_t *s1,
+					const rspamd_ftok_t *s2)
 {
 	g_assert(s1 != NULL && s2 != NULL);
 
@@ -429,7 +432,7 @@ rspamd_ftok_cstr(const rspamd_ftok_t *s)
 }
 
 gboolean
-rspamd_ftok_cstr_equal(const rspamd_ftok_t *s, const gchar *pat,
+rspamd_ftok_cstr_equal(const rspamd_ftok_t *s, const char *pat,
 					   gboolean icase)
 {
 	gsize slen;
@@ -449,10 +452,10 @@ rspamd_ftok_cstr_equal(const rspamd_ftok_t *s, const gchar *pat,
 	return (rspamd_ftok_cmp(s, &srch) == 0);
 }
 
-gchar *
+char *
 rspamd_ftokdup(const rspamd_ftok_t *src)
 {
-	gchar *newstr;
+	char *newstr;
 
 	if (src == NULL) {
 		return NULL;
@@ -465,10 +468,10 @@ rspamd_ftokdup(const rspamd_ftok_t *src)
 	return newstr;
 }
 
-gchar *
+char *
 rspamd_fstringdup(const rspamd_fstring_t *src)
 {
-	gchar *newstr;
+	char *newstr;
 
 	if (src == NULL) {
 		return NULL;
