@@ -27,6 +27,7 @@
 #include "contrib/mumhash/mum.h"
 
 #include <math.h>
+#include <netdb.h>
 
 
 struct upstream_inet_addr_entry {
@@ -61,6 +62,10 @@ struct upstream {
 	ev_timer ev;
 	double last_fail;
 	double last_resolve;
+	/* Probe/half-open state */
+	double next_probe_at;
+	double probe_backoff;
+	unsigned int half_open_inflight;
 	gpointer ud;
 	enum rspamd_upstream_flag flags;
 	struct upstream_list *ls;
@@ -88,6 +93,8 @@ struct upstream_limits {
 	double dns_timeout;
 	double lazy_resolve_time;
 	double resolve_min_interval;
+	double probe_max_backoff;
+	double probe_jitter;
 	unsigned int max_errors;
 	unsigned int dns_retransmits;
 };
@@ -162,6 +169,10 @@ static const unsigned int default_dns_retransmits = DEFAULT_DNS_RETRANSMITS;
 static const double default_lazy_resolve_time = DEFAULT_LAZY_RESOLVE_TIME;
 #define DEFAULT_RESOLVE_MIN_INTERVAL 60.0
 static const double default_resolve_min_interval = DEFAULT_RESOLVE_MIN_INTERVAL;
+#define DEFAULT_PROBE_MAX_BACKOFF 600.0
+static const double default_probe_max_backoff = DEFAULT_PROBE_MAX_BACKOFF;
+#define DEFAULT_PROBE_JITTER 0.3
+static const double default_probe_jitter = DEFAULT_PROBE_JITTER;
 
 static const struct upstream_limits default_limits = {
 	.revive_time = DEFAULT_REVIVE_TIME,
@@ -172,6 +183,8 @@ static const struct upstream_limits default_limits = {
 	.max_errors = DEFAULT_MAX_ERRORS,
 	.lazy_resolve_time = DEFAULT_LAZY_RESOLVE_TIME,
 	.resolve_min_interval = DEFAULT_RESOLVE_MIN_INTERVAL,
+	.probe_max_backoff = DEFAULT_PROBE_MAX_BACKOFF,
+	.probe_jitter = DEFAULT_PROBE_JITTER,
 };
 
 static void rspamd_upstream_lazy_resolve_cb(struct ev_loop *, ev_timer *, int);
@@ -205,6 +218,12 @@ void rspamd_upstreams_library_config(struct rspamd_config *cfg,
 	if (cfg->upstream_resolve_min_interval) {
 		ctx->limits.resolve_min_interval = cfg->upstream_resolve_min_interval;
 	}
+	if (cfg->upstream_probe_max_backoff) {
+		ctx->limits.probe_max_backoff = cfg->upstream_probe_max_backoff;
+	}
+	if (cfg->upstream_probe_jitter) {
+		ctx->limits.probe_jitter = cfg->upstream_probe_jitter;
+	}
 
 	/* Some sanity checks */
 	if (ctx->limits.resolve_min_interval > ctx->limits.revive_time) {
@@ -226,7 +245,8 @@ void rspamd_upstreams_library_config(struct rspamd_config *cfg,
 		while (cur) {
 			upstream = cur->data;
 			if (!ev_can_stop(&upstream->ev) && upstream->ls &&
-				!(upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE)) {
+				!((upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE) ||
+				  (upstream->flags & RSPAMD_UPSTREAM_FLAG_DNS))) {
 				double when;
 
 				if (upstream->flags & RSPAMD_UPSTREAM_FLAG_SRV_RESOLVE) {
@@ -341,7 +361,8 @@ rspamd_upstream_set_active(struct upstream_list *ls, struct upstream *upstream)
 	upstream->active_idx = ls->alive->len - 1;
 
 	if (upstream->ctx && upstream->ctx->configured &&
-		!(upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE)) {
+		!((upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE) ||
+		  (upstream->flags & RSPAMD_UPSTREAM_FLAG_DNS))) {
 
 		if (ev_can_stop(&upstream->ev)) {
 			ev_timer_stop(upstream->ctx->event_loop, &upstream->ev);
@@ -654,6 +675,81 @@ rspamd_upstream_resolve_addrs(const struct upstream_list *ls,
 							  struct upstream *upstream)
 {
 
+	if ((upstream->flags & RSPAMD_UPSTREAM_FLAG_DNS)) {
+		/* For DNS upstreams: resolve synchronously using getaddrinfo if name */
+		if (upstream->name[0] != '/') {
+			/* If marked NORESOLVE at init (numeric address), keep old behaviour */
+			if (upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE) {
+				return;
+			}
+
+			/* Extract host part without port */
+			char dns_name[253 + 1];
+			const char *semicolon_pos = strchr(upstream->name, ':');
+
+			if (semicolon_pos != NULL && semicolon_pos > upstream->name) {
+				if (sizeof(dns_name) > (size_t) (semicolon_pos - upstream->name)) {
+					rspamd_strlcpy(dns_name, upstream->name,
+								   semicolon_pos - upstream->name + 1);
+				}
+				else {
+					msg_err_upstream("internal error: upstream name is larger than max DNS name: %s",
+									 upstream->name);
+					rspamd_strlcpy(dns_name, upstream->name, sizeof(dns_name));
+				}
+			}
+			else {
+				rspamd_strlcpy(dns_name, upstream->name, sizeof(dns_name));
+			}
+
+			/* Use saved port from current address */
+			unsigned int port = 0;
+			if (upstream->addrs.addr && upstream->addrs.addr->len > 0) {
+				struct upstream_addr_elt *addr_elt = g_ptr_array_index(upstream->addrs.addr, 0);
+				port = rspamd_inet_address_get_port(addr_elt->addr);
+			}
+
+			struct addrinfo hints, *res = NULL, *cur;
+			memset(&hints, 0, sizeof(hints));
+			hints.ai_socktype = SOCK_STREAM;
+			hints.ai_family = AF_UNSPEC;
+			hints.ai_flags = AI_NUMERICSERV;
+
+			char portbuf[8];
+			if (port == 0) {
+				/* Fallback to default 53 if unknown */
+				rspamd_strlcpy(portbuf, "53", sizeof(portbuf));
+			}
+			else {
+				rspamd_snprintf(portbuf, sizeof(portbuf), "%ud", port);
+			}
+
+			int gr = getaddrinfo(dns_name, portbuf, &hints, &res);
+			if (gr == 0 && res != NULL) {
+				RSPAMD_UPSTREAM_LOCK(upstream);
+				struct upstream_inet_addr_entry *up_ent;
+				for (cur = res; cur != NULL; cur = cur->ai_next) {
+					rspamd_inet_addr_t *na = rspamd_inet_address_from_sa(cur->ai_addr, cur->ai_addrlen);
+					if (na == NULL) {
+						continue;
+					}
+					rspamd_inet_address_set_port(na, port);
+					up_ent = g_malloc0(sizeof(*up_ent));
+					up_ent->addr = na;
+					up_ent->priority = 0;
+					LL_PREPEND(upstream->new_addrs, up_ent);
+				}
+				RSPAMD_UPSTREAM_UNLOCK(upstream);
+
+				freeaddrinfo(res);
+
+				rspamd_upstream_update_addrs(upstream);
+			}
+		}
+
+		return;
+	}
+
 	if (upstream->ctx->res != NULL &&
 		upstream->ctx->configured &&
 		upstream->dns_requests == 0 &&
@@ -790,6 +886,20 @@ rspamd_upstream_set_inactive(struct upstream_list *ls, struct upstream *upstream
 
 		msg_debug_upstream("mark upstream %s inactive; revive in %.0f seconds",
 						   upstream->name, ntim);
+		/* Initialize probe scheduling */
+		if (upstream->probe_backoff <= 0) {
+			upstream->probe_backoff = ls->limits->revive_time;
+		}
+		if (upstream->ctx && upstream->ctx->event_loop) {
+			upstream->next_probe_at = ev_now(upstream->ctx->event_loop) +
+									  rspamd_time_jitter(upstream->probe_backoff,
+														 upstream->probe_backoff * ls->limits->probe_jitter);
+		}
+		else {
+			double now = rspamd_get_ticks(FALSE);
+			upstream->next_probe_at = now + rspamd_time_jitter(upstream->probe_backoff,
+															   upstream->probe_backoff * ls->limits->probe_jitter);
+		}
 		ev_timer_init(&upstream->ev, rspamd_upstream_revive_cb, ntim, 0);
 		upstream->ev.data = upstream;
 
@@ -923,6 +1033,18 @@ void rspamd_upstream_fail(struct upstream *upstream,
 			}
 		}
 
+		/* If this was a half-open probe, schedule next probe with backoff */
+		if (upstream->half_open_inflight > 0) {
+			double now = upstream->ctx && upstream->ctx->event_loop ? ev_now(upstream->ctx->event_loop) : rspamd_get_ticks(FALSE);
+			if (upstream->probe_backoff <= 0) {
+				upstream->probe_backoff = upstream->ls->limits->revive_time;
+			}
+			upstream->probe_backoff = MIN(upstream->probe_backoff * 2.0, upstream->ls->limits->probe_max_backoff);
+			upstream->next_probe_at = now + rspamd_time_jitter(upstream->probe_backoff,
+															   upstream->probe_backoff * upstream->ls->limits->probe_jitter);
+			upstream->half_open_inflight = 0;
+		}
+
 		RSPAMD_UPSTREAM_UNLOCK(upstream);
 	}
 }
@@ -933,6 +1055,18 @@ void rspamd_upstream_ok(struct upstream *upstream)
 	struct upstream_list_watcher *w;
 
 	RSPAMD_UPSTREAM_LOCK(upstream);
+	/* Success handling */
+	if (upstream->half_open_inflight > 0) {
+		/* Successful probe: mark alive and reset backoff */
+		upstream->half_open_inflight = 0;
+		upstream->probe_backoff = upstream->ls ? upstream->ls->limits->revive_time : default_revive_time;
+		upstream->next_probe_at = 0;
+		if (upstream->ls && upstream->active_idx == -1) {
+			/* Activate this upstream */
+			rspamd_upstream_set_active(upstream->ls, upstream);
+		}
+	}
+
 	if (upstream->errors > 0 && upstream->active_idx != -1 && upstream->ls) {
 		/* We touch upstream if and only if it is active */
 		msg_debug_upstream("reset errors on upstream %s (was %ud)", upstream->name, upstream->errors);
@@ -1207,7 +1341,56 @@ rspamd_upstreams_add_upstream(struct upstream_list *ups, const char *str,
 			}
 		}
 		else {
-			g_ptr_array_free(addrs, TRUE);
+			/* Not numeric: resolve synchronously and add all IPs */
+			struct addrinfo hints, *res = NULL, *cur;
+			char hostbuf[256];
+			const char *colon = strchr(str, ':');
+			char portbuf[8];
+			unsigned int portnum = def_port;
+
+			if (colon != NULL && colon > str) {
+				size_t hlen = MIN((size_t) (colon - str), sizeof(hostbuf) - 1);
+				rspamd_strlcpy(hostbuf, str, hlen + 1);
+				if (colon[1] != '\0') {
+					portnum = strtoul(colon + 1, NULL, 10);
+				}
+			}
+			else {
+				rspamd_strlcpy(hostbuf, str, sizeof(hostbuf));
+			}
+
+			rspamd_snprintf(portbuf, sizeof(portbuf), "%ud", portnum);
+			memset(&hints, 0, sizeof(hints));
+			hints.ai_socktype = SOCK_STREAM;
+			hints.ai_family = AF_UNSPEC;
+			hints.ai_flags = AI_NUMERICSERV;
+
+			if (getaddrinfo(hostbuf, portbuf, &hints, &res) == 0 && res != NULL) {
+				for (cur = res; cur != NULL; cur = cur->ai_next) {
+					rspamd_inet_addr_t *na = rspamd_inet_address_from_sa(cur->ai_addr, cur->ai_addrlen);
+					if (na == NULL) {
+						continue;
+					}
+					rspamd_inet_address_set_port(na, portnum);
+					g_ptr_array_add(addrs, na);
+				}
+				freeaddrinfo(res);
+				if (ups->ctx) {
+					rspamd_mempool_add_destructor(ups->ctx->pool,
+												  (rspamd_mempool_destruct_t) rspamd_ptr_array_free_hard,
+												  addrs);
+				}
+				if (ups->ctx) {
+					upstream->name = rspamd_mempool_strdup(ups->ctx->pool, str);
+				}
+				else {
+					upstream->name = g_strdup(str);
+				}
+				ret = RSPAMD_PARSE_ADDR_RESOLVED;
+			}
+			else {
+				g_ptr_array_free(addrs, TRUE);
+			}
 		}
 
 		break;
@@ -1616,10 +1799,42 @@ rspamd_upstream_get_common(struct upstream_list *ups,
 
 	RSPAMD_UPSTREAM_LOCK(ups);
 	if (ups->alive->len == 0) {
-		/* We have no upstreams alive */
-		msg_warn("there are no alive upstreams left for %s, revive all of them",
-				 ups->ups_line);
-		g_ptr_array_foreach(ups->ups, rspamd_upstream_restore_cb, ups);
+		/* Probe mode: find the earliest probe-ready upstream and allow one inflight */
+		double now = ups->ctx && ups->ctx->event_loop ? ev_now(ups->ctx->event_loop) : rspamd_get_ticks(FALSE);
+		struct upstream *candidate = NULL;
+		double min_probe = HUGE_VAL;
+
+		for (unsigned int i = 0; i < ups->ups->len; i++) {
+			struct upstream *cur = g_ptr_array_index(ups->ups, i);
+			if (cur->active_idx >= 0 || (except && cur == except)) {
+				continue;
+			}
+
+			if (cur->next_probe_at == 0) {
+				/* Initialize probe schedule based on revive_time */
+				cur->probe_backoff = cur->probe_backoff > 0 ? cur->probe_backoff : ups->limits->revive_time;
+				cur->next_probe_at = now + rspamd_time_jitter(cur->probe_backoff,
+															  cur->probe_backoff * ups->limits->probe_jitter);
+			}
+
+			if (cur->next_probe_at <= now && cur->half_open_inflight == 0) {
+				candidate = cur;
+				break;
+			}
+
+			if (cur->next_probe_at < min_probe) {
+				min_probe = cur->next_probe_at;
+			}
+		}
+
+		if (candidate) {
+			candidate->half_open_inflight = 1; /* allow one request */
+			up = candidate;
+		}
+
+		RSPAMD_UPSTREAM_UNLOCK(ups);
+
+		return up; /* can be NULL if not ready yet */
 	}
 	RSPAMD_UPSTREAM_UNLOCK(ups);
 
