@@ -23,10 +23,10 @@
 extern "C" {
 #endif
 
-gint lua_compress_zstd_compress(lua_State *L);
-gint lua_compress_zstd_decompress(lua_State *L);
-gint lua_compress_zlib_compress(lua_State *L);
-gint lua_compress_zlib_decompress(lua_State *L, bool is_gzip);
+int lua_compress_zstd_compress(lua_State *L);
+int lua_compress_zstd_decompress(lua_State *L);
+int lua_compress_zlib_compress(lua_State *L);
+int lua_compress_zlib_decompress(lua_State *L, bool is_gzip);
 
 void luaopen_compress(lua_State *L);
 
