@@ -110,6 +110,12 @@ LUA_FUNCTION_DEF(html, foreach_tag);
  * @return
  */
 LUA_FUNCTION_DEF(html, get_invisible);
+/***
+ * @method html:get_features()
+ * Returns aggregated HTML features as a Lua table
+ * @return {table} features table
+ */
+LUA_FUNCTION_DEF(html, get_features);
 
 static const struct luaL_reg htmllib_m[] = {
 	LUA_INTERFACE_DEF(html, has_tag),
@@ -117,6 +123,7 @@ static const struct luaL_reg htmllib_m[] = {
 	LUA_INTERFACE_DEF(html, get_images),
 	LUA_INTERFACE_DEF(html, foreach_tag),
 	LUA_INTERFACE_DEF(html, get_invisible),
+	LUA_INTERFACE_DEF(html, get_features),
 	{"__tostring", rspamd_lua_class_tostring},
 	{NULL, NULL}};
 
@@ -179,6 +186,44 @@ LUA_FUNCTION_DEF(html_tag, get_style);
  */
 LUA_FUNCTION_DEF(html_tag, get_attribute);
 
+/***
+ * @method html_tag:get_all_attributes()
+ * Returns table of all attributes for the element
+ * @return {table} table with attribute names as keys and values as strings
+ */
+LUA_FUNCTION_DEF(html_tag, get_all_attributes);
+
+/***
+ * @method html_tag:get_unknown_attributes()
+ * Returns table of unknown/unrecognized attributes for the element
+ * @return {table} table with unknown attribute names as keys and values as strings
+ */
+LUA_FUNCTION_DEF(html_tag, get_unknown_attributes);
+
+/***
+ * @method html_tag:get_children()
+ * Returns array of child tags for the element
+ * @return {table} array of child html_tag objects
+ */
+LUA_FUNCTION_DEF(html_tag, get_children);
+
+/***
+ * @method html_tag:has_attribute(name)
+ * Checks if element has the specified attribute
+ * @param {string} name attribute name to check
+ * @return {boolean} true if attribute exists
+ */
+LUA_FUNCTION_DEF(html_tag, has_attribute);
+
+/***
+ * @method html_tag:get_numeric_attribute(name)
+ * Returns numeric value of attribute (if supported and parseable)
+ * Works for attributes like width, height, font-size, etc.
+ * @param {string} name attribute name
+ * @return {number|nil} numeric value or nil if not numeric/parseable
+ */
+LUA_FUNCTION_DEF(html_tag, get_numeric_attribute);
+
 static const struct luaL_reg taglib_m[] = {
 	LUA_INTERFACE_DEF(html_tag, get_type),
 	LUA_INTERFACE_DEF(html_tag, get_extra),
@@ -188,6 +233,11 @@ static const struct luaL_reg taglib_m[] = {
 	LUA_INTERFACE_DEF(html_tag, get_content_length),
 	LUA_INTERFACE_DEF(html_tag, get_style),
 	LUA_INTERFACE_DEF(html_tag, get_attribute),
+	LUA_INTERFACE_DEF(html_tag, get_all_attributes),
+	LUA_INTERFACE_DEF(html_tag, get_unknown_attributes),
+	LUA_INTERFACE_DEF(html_tag, get_children),
+	LUA_INTERFACE_DEF(html_tag, has_attribute),
+	LUA_INTERFACE_DEF(html_tag, get_numeric_attribute),
 	{"__tostring", rspamd_lua_class_tostring},
 	{NULL, NULL}};
 
@@ -507,6 +557,132 @@ lua_html_get_invisible(lua_State *L)
 	return 1;
 }
 
+static int
+lua_html_get_features(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	auto *hc = lua_check_html(L, 1);
+
+	if (hc == NULL) {
+		lua_pushnil(L);
+		return 1;
+	}
+
+	const struct rspamd_html_features *hf = &hc->features;
+
+	/* Top-level table */
+	lua_createtable(L, 0, 16);
+
+	/* version */
+	lua_pushstring(L, "version");
+	lua_pushinteger(L, hf->version);
+	lua_settable(L, -3);
+
+	/* links subtable */
+	lua_pushstring(L, "links");
+	lua_createtable(L, 0, 20);
+
+#define PUSH_FIELD(name)                \
+	lua_pushstring(L, #name);           \
+	lua_pushinteger(L, hf->links.name); \
+	lua_settable(L, -3)
+
+	PUSH_FIELD(total_links);
+	PUSH_FIELD(affiliated_links);
+	PUSH_FIELD(unaffiliated_links);
+	PUSH_FIELD(confusable_like_from_links);
+	PUSH_FIELD(punycode_links);
+	PUSH_FIELD(ip_links);
+	PUSH_FIELD(port_links);
+	PUSH_FIELD(long_query_links);
+	PUSH_FIELD(trackerish_links);
+	PUSH_FIELD(display_mismatch_links);
+	PUSH_FIELD(js_scheme_links);
+	PUSH_FIELD(data_scheme_links);
+	PUSH_FIELD(mailto_links);
+	PUSH_FIELD(http_links);
+	PUSH_FIELD(query_links);
+	PUSH_FIELD(same_etld1_links);
+	PUSH_FIELD(domains_total);
+	PUSH_FIELD(max_links_single_domain);
+
+#undef PUSH_FIELD
+
+	/* set links */
+	lua_settable(L, -3);
+
+	/* forms */
+	lua_pushstring(L, "forms_count");
+	lua_pushinteger(L, hf->forms_count);
+	lua_settable(L, -3);
+	lua_pushstring(L, "forms_post_unaffiliated");
+	lua_pushinteger(L, hf->forms_post_unaffiliated);
+	lua_settable(L, -3);
+	lua_pushstring(L, "forms_post_affiliated");
+	lua_pushinteger(L, hf->forms_post_affiliated);
+	lua_settable(L, -3);
+	lua_pushstring(L, "has_password_input");
+	lua_pushinteger(L, hf->has_password_input);
+	lua_settable(L, -3);
+
+	/* images */
+	lua_pushstring(L, "images_total");
+	lua_pushinteger(L, hf->images_total);
+	lua_settable(L, -3);
+	lua_pushstring(L, "images_external");
+	lua_pushinteger(L, hf->images_external);
+	lua_settable(L, -3);
+	lua_pushstring(L, "images_data");
+	lua_pushinteger(L, hf->images_data);
+	lua_settable(L, -3);
+	lua_pushstring(L, "images_tiny_external");
+	lua_pushinteger(L, hf->images_tiny_external);
+	lua_settable(L, -3);
+
+	/* dom */
+	lua_pushstring(L, "tags_count");
+	lua_pushinteger(L, hf->tags_count);
+	lua_settable(L, -3);
+	lua_pushstring(L, "max_dom_depth");
+	lua_pushinteger(L, hf->max_dom_depth);
+	lua_settable(L, -3);
+
+	/* visibility/text */
+	lua_pushstring(L, "text_visible");
+	lua_pushinteger(L, hf->text_visible);
+	lua_settable(L, -3);
+	lua_pushstring(L, "text_hidden");
+	lua_pushinteger(L, hf->text_hidden);
+	lua_settable(L, -3);
+	lua_pushstring(L, "text_transparent");
+	lua_pushinteger(L, hf->text_transparent);
+	lua_settable(L, -3);
+	lua_pushstring(L, "blocks_hidden");
+	lua_pushinteger(L, hf->blocks_hidden);
+	lua_settable(L, -3);
+	lua_pushstring(L, "blocks_transparent");
+	lua_pushinteger(L, hf->blocks_transparent);
+	lua_settable(L, -3);
+	lua_pushstring(L, "offscreen_blocks");
+	lua_pushinteger(L, hf->offscreen_blocks);
+	lua_settable(L, -3);
+
+	/* meta/obfuscation */
+	lua_pushstring(L, "meta_refresh");
+	lua_pushinteger(L, hf->meta_refresh);
+	lua_settable(L, -3);
+	lua_pushstring(L, "meta_refresh_urls");
+	lua_pushinteger(L, hf->meta_refresh_urls);
+	lua_settable(L, -3);
+
+	/* flags */
+	lua_pushstring(L, "flags");
+	lua_pushinteger(L, hf->flags);
+	lua_settable(L, -3);
+
+	return 1;
+}
+
 static int
 lua_html_tag_get_type(lua_State *L)
 {
@@ -703,6 +879,29 @@ lua_html_tag_get_style(lua_State *L)
 	return 1;
 }
 
+static int
+lua_html_tag_get_all_attributes(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct lua_html_tag *ltag = lua_check_html_tag(L, 1);
+
+	if (ltag) {
+		auto all_attrs = ltag->tag->get_all_attributes();
+		lua_createtable(L, 0, all_attrs.size());
+
+		for (const auto &[name, value]: all_attrs) {
+			lua_pushlstring(L, name.data(), name.size());
+			lua_pushlstring(L, value.data(), value.size());
+			lua_settable(L, -3);
+		}
+	}
+	else {
+		return luaL_error(L, "invalid arguments");
+	}
+
+	return 1;
+}
+
 static int
 lua_html_tag_get_attribute(lua_State *L)
 {
@@ -712,8 +911,7 @@ lua_html_tag_get_attribute(lua_State *L)
 	const char *attr_name = luaL_checklstring(L, 2, &slen);
 
 	if (ltag && attr_name) {
-		auto maybe_attr = ltag->tag->find_component(
-			rspamd::html::html_component_from_string({attr_name, slen}));
+		auto maybe_attr = ltag->tag->find_component_by_name({attr_name, slen});
 
 		if (maybe_attr) {
 			lua_pushlstring(L, maybe_attr->data(), maybe_attr->size());
@@ -729,6 +927,206 @@ lua_html_tag_get_attribute(lua_State *L)
 	return 1;
 }
 
+static int
+lua_html_tag_get_unknown_attributes(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct lua_html_tag *ltag = lua_check_html_tag(L, 1);
+
+	if (ltag) {
+		auto unknown_attrs = ltag->tag->get_unknown_components();
+		lua_createtable(L, 0, unknown_attrs.size());
+
+		for (const auto &[name, value]: unknown_attrs) {
+			lua_pushlstring(L, name.data(), name.size());
+			lua_pushlstring(L, value.data(), value.size());
+			lua_settable(L, -3);
+		}
+	}
+	else {
+		return luaL_error(L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+static int
+lua_html_tag_get_children(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct lua_html_tag *ltag = lua_check_html_tag(L, 1);
+
+	if (ltag) {
+		lua_createtable(L, ltag->tag->children.size(), 0);
+
+		for (int i = 0; i < ltag->tag->children.size(); i++) {
+			auto *child_tag = static_cast<lua_html_tag *>(lua_newuserdata(L, sizeof(lua_html_tag)));
+			child_tag->tag = ltag->tag->children[i];
+			child_tag->html = ltag->html;
+			rspamd_lua_setclass(L, rspamd_html_tag_classname, -1);
+			lua_rawseti(L, -2, i + 1);
+		}
+	}
+	else {
+		return luaL_error(L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+static int
+lua_html_tag_has_attribute(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct lua_html_tag *ltag = lua_check_html_tag(L, 1);
+	gsize slen;
+	const char *attr_name = luaL_checklstring(L, 2, &slen);
+
+	if (ltag && attr_name) {
+		auto maybe_attr = ltag->tag->find_component_by_name({attr_name, slen});
+		lua_pushboolean(L, maybe_attr.has_value());
+	}
+	else {
+		return luaL_error(L, "invalid arguments");
+	}
+
+	return 1;
+}
+
+static int
+lua_html_tag_get_numeric_attribute(lua_State *L)
+{
+	LUA_TRACE_POINT;
+	struct lua_html_tag *ltag = lua_check_html_tag(L, 1);
+	gsize slen;
+	const char *attr_name = luaL_checklstring(L, 2, &slen);
+
+	if (ltag && attr_name) {
+		std::string_view name_view{attr_name, slen};
+
+		// Check for numeric components
+		if (name_view == "width") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_width>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "height") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_height>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "size") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_size>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "font-size") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_font_size>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "line-height") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_line_height>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "border-width") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_border_width>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "opacity") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_opacity>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushnumber(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "min-width") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_min_width>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "max-width") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_max_width>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "min-height") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_min_height>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "max-height") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_max_height>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "cellpadding") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_cellpadding>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "cellspacing") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_cellspacing>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+		else if (name_view == "tabindex") {
+			if (auto comp = ltag->tag->find_component<rspamd::html::html_component_tabindex>()) {
+				if (auto numeric_val = comp.value()->get_numeric_value()) {
+					lua_pushinteger(L, numeric_val.value());
+					return 1;
+				}
+			}
+		}
+
+		lua_pushnil(L);
+	}
+	else {
+		return luaL_error(L, "invalid arguments");
+	}
+
+	return 1;
+}
+
 void luaopen_html(lua_State *L)
 {
 	rspamd_lua_new_class(L, rspamd_html_classname, htmllib_m);
