@@ -19,6 +19,7 @@
 
 #include "contrib/hiredis/hiredis.h"
 #include "contrib/hiredis/async.h"
+#include "redis_pool.h"
 
 #define REDIS_DEFAULT_TIMEOUT 1.0
 
@@ -887,13 +888,19 @@ rspamd_lua_redis_prepare_connection(lua_State *L, int *pcbref, gboolean is_async
 	struct rspamd_task *task = NULL;
 	const char *host = NULL;
 	const char *username = NULL, *password = NULL, *dbname = NULL, *log_tag = NULL;
+	struct rspamd_redis_tls_opts tls_opts;
 	int cbref = -1;
+	/* Track postponed TLS values on Lua stack and table index */
+	int tls_items = 0;
+	int tls_tbl_idx = 0;
 	struct rspamd_config *cfg = NULL;
 	struct rspamd_async_session *session = NULL;
 	struct ev_loop *ev_base = NULL;
 	gboolean ret = FALSE;
 	unsigned int flags = 0;
 
+	memset(&tls_opts, 0, sizeof(tls_opts));
+
 	if (lua_istable(L, 1)) {
 		/* Table version */
 		lua_pushvalue(L, 1);
@@ -1009,14 +1016,64 @@ rspamd_lua_redis_prepare_connection(lua_State *L, int *pcbref, gboolean is_async
 		}
 		lua_pop(L, 1);
 
+		/* TLS options (optional). Postpone pops to keep values alive */
+		tls_tbl_idx = lua_gettop(L);
+
+		lua_pushstring(L, "ssl");
+		lua_gettable(L, tls_tbl_idx);
+		if (!!lua_toboolean(L, -1)) {
+			tls_opts.use_tls = true;
+		}
+		tls_items++;
+
+		lua_pushstring(L, "no_ssl_verify");
+		lua_gettable(L, tls_tbl_idx);
+		if (!!lua_toboolean(L, -1)) {
+			tls_opts.no_ssl_verify = true;
+		}
+		tls_items++;
+
+		lua_pushstring(L, "ssl_ca");
+		lua_gettable(L, tls_tbl_idx);
+		if (lua_type(L, -1) == LUA_TSTRING) {
+			tls_opts.ca_file = lua_tostring(L, -1);
+		}
+		tls_items++;
+
+		lua_pushstring(L, "ssl_ca_dir");
+		lua_gettable(L, tls_tbl_idx);
+		if (lua_type(L, -1) == LUA_TSTRING) {
+			tls_opts.ca_dir = lua_tostring(L, -1);
+		}
+		tls_items++;
+
+		lua_pushstring(L, "ssl_cert");
+		lua_gettable(L, tls_tbl_idx);
+		if (lua_type(L, -1) == LUA_TSTRING) {
+			tls_opts.cert_file = lua_tostring(L, -1);
+		}
+		tls_items++;
+
+		lua_pushstring(L, "ssl_key");
+		lua_gettable(L, tls_tbl_idx);
+		if (lua_type(L, -1) == LUA_TSTRING) {
+			tls_opts.key_file = lua_tostring(L, -1);
+		}
+		tls_items++;
+
+		lua_pushstring(L, "sni");
+		lua_gettable(L, tls_tbl_idx);
+		if (lua_type(L, -1) == LUA_TSTRING) {
+			tls_opts.sni = lua_tostring(L, -1);
+		}
+		tls_items++;
+
 		lua_pushstring(L, "no_pool");
-		lua_gettable(L, -2);
+		lua_gettable(L, tls_tbl_idx);
 		if (!!lua_toboolean(L, -1)) {
 			flags |= LUA_REDIS_NO_POOL;
 		}
-		lua_pop(L, 1);
-
-		lua_pop(L, 1); /* table */
+		tls_items++;
 
 		if (session && rspamd_session_blocked(session)) {
 			msg_err_task_check("Session is being destroying");
@@ -1070,10 +1127,21 @@ rspamd_lua_redis_prepare_connection(lua_State *L, int *pcbref, gboolean is_async
 
 	if (ret) {
 		ud->terminated = 0;
-		ud->ctx = rspamd_redis_pool_connect(ud->pool,
-											dbname, username, password,
-											rspamd_inet_address_to_string(addr->addr),
-											rspamd_inet_address_get_port(addr->addr));
+		ud->ctx = rspamd_redis_pool_connect_ext(ud->pool,
+												dbname, username, password,
+												rspamd_inet_address_to_string(addr->addr),
+												rspamd_inet_address_get_port(addr->addr),
+												&tls_opts);
+
+		/* Pop postponed TLS values and the table itself */
+		if (tls_items > 0) {
+			lua_pop(L, tls_items);
+			tls_items = 0;
+		}
+		if (tls_tbl_idx != 0) {
+			lua_pop(L, 1);
+			tls_tbl_idx = 0;
+		}
 
 		if (ip) {
 			rspamd_inet_address_free(ip);
@@ -1612,9 +1680,9 @@ lua_redis_exec(lua_State *L)
 		int replies_pending = g_queue_get_length(ctx->replies);
 
 		msg_debug_lua_redis("execute pending commands for %p; commands pending = %d; replies pending = %d",
-			ctx,
-			ctx->cmds_pending,
-			replies_pending);
+							ctx,
+							ctx->cmds_pending,
+							replies_pending);
 
 		if (ctx->cmds_pending == 0 && replies_pending == 0) {
 			lua_pushstring(L, "No pending commands to execute");
