@@ -135,7 +135,7 @@ local function clusterting_filter_cb(task, rule)
   end
 
   if not cluster_selector or not source_selector then
-    rspamd_logger.debugm(N, task, 'skip rule %s, selectors: source="%s", cluster="%s"',
+    lua_util.debugm(N, task, 'skip rule %s, selectors: source="%s", cluster="%s"',
         rule.name, source_selector, cluster_selector)
     return
   end
@@ -153,7 +153,7 @@ local function clusterting_filter_cb(task, rule)
       -- We have seen this element in ham mostly, so subtract average it from the size score
       final_score = math.min(1.0, size_score - cluster_score / cur_elts)
     end
-    rspamd_logger.debugm(N, task,
+    lua_util.debugm(N, task,
         'processed rule %s, selectors: source="%s", cluster="%s"; data: %s elts, %s score, %s elt score',
         rule.name, source_selector, cluster_selector, cur_elts, total_score, element_score)
     if final_score > 0.1 then
@@ -205,7 +205,7 @@ local function clusterting_idempotent_cb(task, rule)
   elseif verdict == 'junk' then
     score = rule.junk_mult
   else
-    rspamd_logger.debugm(N, task, 'skip rule %s, verdict=%s',
+    lua_util.debugm(N, task, 'skip rule %s, verdict=%s',
         rule.name, verdict)
     return
   end
@@ -218,7 +218,7 @@ local function clusterting_idempotent_cb(task, rule)
   end
 
   if not cluster_selector or not source_selector then
-    rspamd_logger.debugm(N, task, 'skip rule %s, selectors: source="%s", cluster="%s"',
+    lua_util.debugm(N, task, 'skip rule %s, selectors: source="%s", cluster="%s"',
         rule.name, source_selector, cluster_selector)
     return
   end
@@ -228,7 +228,7 @@ local function clusterting_idempotent_cb(task, rule)
       rspamd_logger.errx(task, 'got error while getting clustering keys %s: %s',
           source_selector, err)
     else
-      rspamd_logger.debugm(N, task, 'set clustering key for %s: %s{%s} = %s',
+      lua_util.debugm(N, task, 'set clustering key for %s: %s{%s} = %s',
           source_selector, "unknown error")
     end
   end
