@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,13 +30,15 @@ static gboolean modules_state = FALSE;
 static gboolean symbol_groups_only = FALSE;
 static gboolean symbol_full_details = FALSE;
 static gboolean skip_template = FALSE;
-static gchar *config = NULL;
+static char *config = NULL;
+static gboolean local_conf_only = FALSE;
+static gboolean override_conf_only = FALSE;
 extern struct rspamd_main *rspamd_main;
 /* Defined in modules.c */
 extern module_t *modules[];
 extern worker_t *workers[];
 
-static void rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *);
+static void rspamadm_configdump(int argc, char **argv, const struct rspamadm_command *);
 static const char *rspamadm_configdump_help(gboolean full_help, const struct rspamadm_command *);
 
 struct rspamadm_command configdump_command = {
@@ -66,6 +68,8 @@ static GOptionEntry entries[] = {
 	 "Show full symbol details only", NULL},
 	{"skip-template", 'T', 0, G_OPTION_ARG_NONE, &skip_template,
 	 "Do not apply Jinja templates", NULL},
+	{"local", 0, 0, G_OPTION_ARG_NONE, &local_conf_only, "Show only local and override configuration", NULL},
+	{"override", 0, 0, G_OPTION_ARG_NONE, &override_conf_only, "Show only override configuration", NULL},
 	{NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
 
 static const char *
@@ -82,6 +86,8 @@ rspamadm_configdump_help(gboolean full_help, const struct rspamadm_command *cmd)
 				   "-c: config file to test\n"
 				   "-m: show state of modules only\n"
 				   "-h: show help for dumped options\n"
+				   "--local: show only local (and override) configuration\n"
+				   "--override: show only override configuration\n"
 				   "--help: shows available options and commands";
 	}
 	else {
@@ -96,6 +102,84 @@ config_logger(rspamd_mempool_t *pool, gpointer ud)
 {
 }
 
+static ucl_object_t *
+filter_non_default(const ucl_object_t *obj, bool override_only)
+{
+	ucl_object_t *result = NULL;
+	ucl_object_iter_t it = NULL;
+	const ucl_object_t *cur;
+
+	if (obj == NULL) {
+		return NULL;
+	}
+
+	int min_prio = override_only ? 1 : 0;
+
+	if (ucl_object_get_priority(obj) > min_prio) {
+
+		switch (ucl_object_type(obj)) {
+		case UCL_OBJECT:
+			result = ucl_object_typed_new(ucl_object_type(obj));
+
+			while ((cur = ucl_object_iterate(obj, &it, true))) {
+				ucl_object_t *filtered = filter_non_default(cur, override_conf_only);
+				if (filtered) {
+					ucl_object_insert_key(result, filtered, ucl_object_key(cur), cur->keylen, true);
+				}
+			}
+			break;
+		case UCL_ARRAY:
+			result = ucl_object_typed_new(ucl_object_type(obj));
+
+			while ((cur = ucl_object_iterate(obj, &it, true))) {
+				ucl_object_t *filtered = filter_non_default(cur, override_conf_only);
+				if (filtered) {
+					ucl_array_append(result, filtered);
+				}
+			}
+		default:
+			result = ucl_object_ref(obj);
+			break;
+		}
+
+		return result;
+	}
+
+	if (ucl_object_type(obj) == UCL_OBJECT || ucl_object_type(obj) == UCL_ARRAY) {
+		bool has_non_default = false;
+
+		result = ucl_object_typed_new(ucl_object_type(obj));
+		while ((cur = ucl_object_iterate(obj, &it, true))) {
+			ucl_object_t *filtered = filter_non_default(cur, override_only);
+			if (filtered) {
+				has_non_default = true;
+
+				if (ucl_object_type(obj) == UCL_OBJECT) {
+					ucl_object_insert_key(result, filtered,
+										  ucl_object_key(cur), cur->keylen, true);
+				}
+				else if (ucl_object_type(obj) == UCL_ARRAY) {
+					ucl_array_append(result, filtered);
+				}
+				else {
+					g_assert_not_reached();
+				}
+			}
+		}
+
+		/* Avoid empty objects */
+		if (!has_non_default) {
+			ucl_object_unref(result);
+			result = NULL;
+		}
+
+		return result;
+	}
+
+
+	return NULL;
+}
+
 static void
 rspamadm_add_doc_elt(const ucl_object_t *obj, const ucl_object_t *doc_obj,
 					 ucl_object_t *comment_obj)
@@ -234,16 +318,16 @@ rspamadm_dump_section_obj(struct rspamd_config *cfg,
 }
 
 __attribute__((noreturn)) static void
-rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
+rspamadm_configdump(int argc, char **argv, const struct rspamadm_command *cmd)
 {
 	GOptionContext *context;
 	GError *error = NULL;
-	const gchar *confdir;
+	const char *confdir;
 	const ucl_object_t *obj = NULL, *cur, *doc_obj;
 	struct rspamd_config *cfg = rspamd_main->cfg;
 	gboolean ret = TRUE;
 	worker_t **pworker;
-	gint i;
+	int i;
 
 	context = g_option_context_new(
 		"configdump - dumps Rspamd configuration");
@@ -319,7 +403,7 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 			const ucl_object_t *all_symbols_ucl = ucl_object_lookup(cfg->cfg_ucl_obj, "symbols");
 
 			while (g_hash_table_iter_next(&it, &sk, &sv)) {
-				const gchar *sym_name = (const gchar *) sk;
+				const char *sym_name = (const char *) sk;
 				struct rspamd_symbol *s = (struct rspamd_symbol *) sv;
 				ucl_object_t *this_sym_ucl = ucl_object_typed_new(UCL_OBJECT);
 
@@ -364,7 +448,7 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 
 					if (s->groups) {
 						ucl_object_t *add_groups = ucl_object_typed_new(UCL_ARRAY);
-						guint j;
+						unsigned int j;
 						struct rspamd_symbols_group *add_gr;
 						bool has_extra_groups = false;
 
@@ -428,7 +512,7 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 			g_hash_table_iter_init(&it, cfg->groups);
 
 			while (g_hash_table_iter_next(&it, &k, &v)) {
-				const gchar *gr_name = (const gchar *) k;
+				const char *gr_name = (const char *) k;
 				struct rspamd_symbols_group *gr = (struct rspamd_symbols_group *) v;
 				ucl_object_t *gr_ucl = ucl_object_typed_new(UCL_OBJECT);
 
@@ -444,6 +528,9 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 				ucl_object_insert_key(gr_ucl,
 									  ucl_object_fromdouble(gr->max_score),
 									  "max_score", strlen("max_score"), false);
+				ucl_object_insert_key(gr_ucl,
+									  ucl_object_fromdouble(gr->min_score),
+									  "min_score", strlen("min_score"), false);
 				ucl_object_insert_key(gr_ucl,
 									  ucl_object_fromstring(gr->description),
 									  "description", strlen("description"), false);
@@ -456,7 +543,7 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 					ucl_object_t *sym_ucl = ucl_object_typed_new(UCL_OBJECT);
 
 					while (g_hash_table_iter_next(&sit, &sk, &sv)) {
-						const gchar *sym_name = (const gchar *) sk;
+						const char *sym_name = (const char *) sk;
 						struct rspamd_symbol *s = (struct rspamd_symbol *) sv;
 						ucl_object_t *spec_sym = ucl_object_typed_new(UCL_OBJECT);
 
@@ -486,7 +573,7 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 						}
 
 						ucl_object_t *add_groups = ucl_object_typed_new(UCL_ARRAY);
-						guint j;
+						unsigned int j;
 						struct rspamd_symbols_group *add_gr;
 
 						PTR_ARRAY_FOREACH(s->groups, j, add_gr)
@@ -521,7 +608,20 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 
 		/* Output configuration */
 		if (argc == 1) {
-			rspamadm_dump_section_obj(cfg, cfg->cfg_ucl_obj, cfg->doc_strings);
+			const ucl_object_t *output_obj = cfg->cfg_ucl_obj;
+			if (local_conf_only || override_conf_only) {
+				output_obj = filter_non_default(cfg->cfg_ucl_obj, override_conf_only);
+				if (!output_obj) {
+					rspamd_printf("No non-default configuration found\n");
+					exit(EXIT_SUCCESS);
+				}
+			}
+
+			rspamadm_dump_section_obj(cfg, output_obj, cfg->doc_strings);
+
+			if (local_conf_only || override_conf_only) {
+				ucl_object_unref((ucl_object_t *) output_obj);
+			}
 		}
 		else {
 			for (i = 1; i < argc; i++) {
@@ -534,10 +634,18 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 				else {
 					LL_FOREACH(obj, cur)
 					{
+						const ucl_object_t *output_obj = cur;
+						if (local_conf_only || override_conf_only) {
+							output_obj = filter_non_default(cur, override_conf_only);
+							if (!output_obj) {
+								rspamd_printf("No non-default configuration found for section %s\n", argv[i]);
+								continue;
+							}
+						}
 						if (!json && !compact) {
 							rspamd_printf("*** Section %s ***\n", argv[i]);
 						}
-						rspamadm_dump_section_obj(cfg, cur, doc_obj);
+						rspamadm_dump_section_obj(cfg, output_obj, doc_obj);
 
 						if (!json && !compact) {
 							rspamd_printf("\n*** End of section %s ***\n", argv[i]);
@@ -545,6 +653,10 @@ rspamadm_configdump(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 						else {
 							rspamd_printf("\n");
 						}
+
+						if (local_conf_only || override_conf_only) {
+							ucl_object_unref((ucl_object_t *) output_obj);
+						}
 					}
 				}
 			}
