@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2023 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,7 +21,7 @@
 #include "lua/lua_common.h"
 
 static gboolean quiet = FALSE;
-static gchar *config = NULL;
+static char *config = NULL;
 static gboolean strict = FALSE;
 static gboolean skip_template = FALSE;
 extern struct rspamd_main *rspamd_main;
@@ -29,7 +29,7 @@ extern struct rspamd_main *rspamd_main;
 extern module_t *modules[];
 extern worker_t *workers[];
 
-static void rspamadm_configtest(gint argc, gchar **argv,
+static void rspamadm_configtest(int argc, char **argv,
 								const struct rspamadm_command *cmd);
 static const char *rspamadm_configtest_help(gboolean full_help,
 											const struct rspamadm_command *cmd);
@@ -79,15 +79,15 @@ config_logger(rspamd_mempool_t *pool, gpointer ud)
 }
 
 static void
-rspamadm_configtest(gint argc, gchar **argv, const struct rspamadm_command *cmd)
+rspamadm_configtest(int argc, char **argv, const struct rspamadm_command *cmd)
 {
 	GOptionContext *context;
 	GError *error = NULL;
-	const gchar *confdir;
+	const char *confdir;
 	struct rspamd_config *cfg = rspamd_main->cfg;
 	gboolean ret = TRUE;
 	worker_t **pworker;
-	const guint64 *log_cnt;
+	const uint64_t *log_cnt;
 
 	context = g_option_context_new(
 		"configtest - perform configuration file test");
@@ -106,7 +106,7 @@ rspamadm_configtest(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 	g_option_context_free(context);
 
 	if (config == NULL) {
-		static gchar fbuf[PATH_MAX];
+		static char fbuf[PATH_MAX];
 
 		if ((confdir = g_hash_table_lookup(ucl_vars, "CONFDIR")) == NULL) {
 			confdir = RSPAMD_CONFDIR;
@@ -150,6 +150,23 @@ rspamadm_configtest(gint argc, gchar **argv, const struct rspamadm_command *cmd)
 											 FALSE)) {
 			ret = FALSE;
 		}
+
+		if (ret) {
+			if (rspamd_lua_require_function(cfg->lua_state, "lua_cfg_utils", "check_configuration_errors")) {
+				GError *err = NULL;
+
+				if (!rspamd_lua_universal_pcall(cfg->lua_state, -1, G_STRLOC, 1, "", &err)) {
+					msg_err_config("call to lua function failed: %s",
+								   lua_tostring(cfg->lua_state, -1));
+					lua_pop(cfg->lua_state, 2);
+					ret = FALSE;
+				}
+				else {
+					ret = lua_toboolean(cfg->lua_state, -1);
+					lua_pop(cfg->lua_state, 2);
+				}
+			}
+		}
 	}
 
 	if (strict && ret) {
