@@ -1,7 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 import socket
 import sys
+
 import dummy_killer
 
 UDP_IP = "127.0.0.1"
@@ -15,10 +16,11 @@ if __name__ == "__main__":
         port = 5005
     sock = socket.socket(socket.AF_INET, # Internet
                          socket.SOCK_DGRAM) # UDP
+    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     sock.bind((UDP_IP, port))
     dummy_killer.write_pid(PID)
 
     while True:
         data, addr = sock.recvfrom(1024) # buffer size is 1024 bytes
-        print "received message:", data
-        sock.sendto(data, addr)
\ No newline at end of file
+        print("received message:", data)
+        sock.sendto(data, addr)
